searchData={"items":[{"type":"module","title":"ExFTP","doc":"See the [README](https://hexdocs.pm/ex_ftp/readme.html)","ref":"ExFTP.html"},{"type":"module","title":"ExFTP.Auth.BasicAuth","doc":"When **authenticator** is `ExFTP.Auth.BasicAuth`, this authenticator will call out to an HTTP endpoint that implements\n[HTTP Basic Auth](https://en.wikipedia.org/wiki/Basic_access_authentication){:target=\"_blank\"} with the user's\nsupplied credentials.\n\n> #### 🔒 Security {: .warning}\n>\n> `BasicAuth` is not recommended for situations not protected by SSL.\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.BasicAuth.html"},{"type":"module","title":"⚙️ Configuration - ExFTP.Auth.BasicAuth","doc":"*Keys*\n\n* **authenticator**  == `ExFTP.Auth.BasicAuth`\n* **authenticator_config** :: `t:ExFTP.Auth.BasicAuthConfig.t/0`\n\n*Example*\n\n```elixir\n  %{\n    # ... ,\n    authenticator: ExFTP.Auth.BasicAuth,\n    authenticator_config: %{\n      login_url: \"https://httpbin.dev/basic-auth/\",\n      login_method: :get,\n      authenticated_url: \"https://httpbin.dev/hidden-basic-auth/\",\n      authenticated_method: :get,\n      authenticated_ttl_ms: 1000 * 60 * 60\n    }\n  }\n```","ref":"ExFTP.Auth.BasicAuth.html#module-configuration"},{"type":"module","title":"👀 See Also - ExFTP.Auth.BasicAuth","doc":"* `ExFTP.Authenticator`","ref":"ExFTP.Auth.BasicAuth.html#module-see-also"},{"type":"module","title":"📖 Resources - ExFTP.Auth.BasicAuth","doc":"* 📖 [RFC 959 (section-4)](https://www.rfc-editor.org/rfc/rfc959#section-4){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Auth.BasicAuth.html#module-resources"},{"type":"function","title":"ExFTP.Auth.BasicAuth.authenticated?/1","doc":"Determines whether this session is still considered authenticated\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.BasicAuth.html#authenticated?/1"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.BasicAuth.authenticated?/1","doc":"* **authenticator_state** :: `t:ExFTP.Authenticator.authenticator_state/0`","ref":"ExFTP.Auth.BasicAuth.html#authenticated?/1-params"},{"type":"function","title":"🧑‍🍳 Workflow - ExFTP.Auth.BasicAuth.authenticated?/1","doc":"* Reads the **authenticator_config**.\n * If the config has **authenticated_url**,\n   * Calls it using HTTP Basic Auth with username and password provided by the user\n   * If the response is **HTTP 200**, success. Otherwise, no longer authenticated.\n * If the config does not have **authenticated_url**,\n   * investigate the **authenticator_state** for `authenticated: true`","ref":"ExFTP.Auth.BasicAuth.html#authenticated?/1-workflow"},{"type":"function","title":"⤵️ Returns - ExFTP.Auth.BasicAuth.authenticated?/1","doc":"**✅ On Success**\n\n  ```elixir\n  `true` or `false`\n  ```","ref":"ExFTP.Auth.BasicAuth.html#authenticated?/1-returns"},{"type":"function","title":"💻 Examples - ExFTP.Auth.BasicAuth.authenticated?/1","doc":"iex> alias ExFTP.Auth.BasicAuth\n    iex> username = \"jsmith\"\n    iex> password = \"password\"\n    iex> Application.put_env(:ex_ftp, :authenticator, ExFTP.Auth.BasicAuth)\n    iex> Application.put_env(:ex_ftp, :authenticator_config, %{\n    iex>  login_url: \"https://httpbin.dev/basic-auth/\" <> username <> \"/\" <> password,\n    iex>  login_method: :get,\n    iex>  authenticated_url: \"https://httpbin.dev/hidden-basic-auth/\" <> username <> \"/\" <> password,\n    iex>  authenticated_method: :get,\n    iex>  authenticated_ttl_ms: 1000 * 60 * 60\n    iex> })\n    iex> {:ok, state} = BasicAuth.login(password, %{username: username})\n    iex> BasicAuth.authenticated?(state)\n    true","ref":"ExFTP.Auth.BasicAuth.html#authenticated?/1-examples"},{"type":"function","title":"👀 See Also - ExFTP.Auth.BasicAuth.authenticated?/1","doc":"* `t:ExFTP.Auth.BasicAuthConfig.t/0`\n  * `t:ExFTP.Auth.Common.authenticated_url/0`\n  * `t:ExFTP.Auth.Common.authenticated_method/0`\n\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.BasicAuth.html#authenticated?/1-see-also"},{"type":"function","title":"ExFTP.Auth.BasicAuth.login/2","doc":"Requests a login using [HTTP Basic Auth](https://en.wikipedia.org/wiki/Basic_access_authentication){:target=\"_blank\"}\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.BasicAuth.html#login/2"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.BasicAuth.login/2","doc":"* **password** :: `t:ExFTP.Authenticator.password/0`\n  * **authenticator_state** :: `t:ExFTP.Authenticator.authenticator_state/0`","ref":"ExFTP.Auth.BasicAuth.html#login/2-params"},{"type":"function","title":"🧑‍🍳 Workflow - ExFTP.Auth.BasicAuth.login/2","doc":"* Reads the **authenticator_config**.\n * Receives a password from the client (a username was provided earlier)\n * Calls the **login_url** with HTTP Basic Auth\n * If the response is **HTTP 200**, success. Otherwise, bad login.","ref":"ExFTP.Auth.BasicAuth.html#login/2-workflow"},{"type":"function","title":"⤵️ Returns - ExFTP.Auth.BasicAuth.login/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, authenticator_state}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, bad_login}\n  ```","ref":"ExFTP.Auth.BasicAuth.html#login/2-returns"},{"type":"function","title":"💻 Examples - ExFTP.Auth.BasicAuth.login/2","doc":"iex> alias ExFTP.Auth.BasicAuth\n    iex> username = \"jsmith\"\n    iex> password = \"password\"\n    iex> Application.put_env(:ex_ftp, :authenticator, ExFTP.Auth.BasicAuth)\n    iex> Application.put_env(:ex_ftp, :authenticator_config, %{\n    iex>  login_url: \"https://httpbin.dev/basic-auth/\" <> username <> \"/\" <> password,\n    iex>  login_method: :get,\n    iex>  authenticated_url: \"https://httpbin.dev/hidden-basic-auth/\" <> username <> \"/\" <> password,\n    iex>  authenticated_method: :get,\n    iex>  authenticated_ttl_ms: 1000 * 60 * 60\n    iex> })\n    iex> {:ok, _} = BasicAuth.login(password, %{username: username})","ref":"ExFTP.Auth.BasicAuth.html#login/2-examples"},{"type":"function","title":"👀 See Also - ExFTP.Auth.BasicAuth.login/2","doc":"* `t:ExFTP.Auth.BasicAuthConfig.t/0`\n  * `t:ExFTP.Auth.Common.login_url/0`\n  * `t:ExFTP.Auth.Common.login_method/0`\n\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.BasicAuth.html#login/2-see-also"},{"type":"function","title":"ExFTP.Auth.BasicAuth.valid_user?/1","doc":"Always returns `true`.\n\n> #### No performance benefit {: .tip}\n> This method is normally used to short-circuit bad login requests.\n> The performance gain in that short-circuit is negligible for basic auth, so it's not used.","ref":"ExFTP.Auth.BasicAuth.html#valid_user?/1"},{"type":"module","title":"ExFTP.Auth.BearerAuth","doc":"When **authenticator** is `ExFTP.Auth.BearerAuth`, this authenticator will call out to an HTTP endpoint that\nimplements [Bearer Tokens](https://swagger.io/docs/specification/v3_0/authentication/bearer-authentication/){:target=\"_blank\"}\nwith the user's supplied credentials.\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.BearerAuth.html"},{"type":"module","title":"⚙️ Configuration - ExFTP.Auth.BearerAuth","doc":"*Keys*\n\n* **authenticator**  == `ExFTP.Auth.BearerAuth`\n* **authenticator_config** :: `t:ExFTP.Auth.BearerAuthConfig.t/0`\n\n*Example*\n\n```elixir\n  %{\n    # ... ,\n    authenticator: ExFTP.Auth.BearerAuth,\n    authenticator_config: %{\n      login_url: \"https://httpbin.dev/bearer\",\n      login_method: :post,\n      authenticated_url: \"https://httpbin.dev/bearer\",\n      authenticated_method: :post,\n      authenticated_ttl_ms: 1000 * 60 * 60\n    }\n  }\n```","ref":"ExFTP.Auth.BearerAuth.html#module-configuration"},{"type":"module","title":"👀 See Also - ExFTP.Auth.BearerAuth","doc":"* `ExFTP.Authenticator`","ref":"ExFTP.Auth.BearerAuth.html#module-see-also"},{"type":"module","title":"📖 Resources - ExFTP.Auth.BearerAuth","doc":"* 📖 [RFC 959 (section-4)](https://www.rfc-editor.org/rfc/rfc959#section-4){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Auth.BearerAuth.html#module-resources"},{"type":"function","title":"ExFTP.Auth.BearerAuth.authenticated?/1","doc":"Determines whether this session is still considered authenticated\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.BearerAuth.html#authenticated?/1"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.BearerAuth.authenticated?/1","doc":"* **authenticator_state** :: `t:ExFTP.Authenticator.authenticator_state/0`","ref":"ExFTP.Auth.BearerAuth.html#authenticated?/1-params"},{"type":"function","title":"🧑‍🍳 Workflow - ExFTP.Auth.BearerAuth.authenticated?/1","doc":"* Reads the **authenticator_config**.\n * If the config has **authenticated_url**,\n   * Calls it with a bearer token provided by the user in the headers\n   * If the response is **HTTP 200**, success. Otherwise, no longer authenticated.\n * If the config does not have **authenticated_url**,\n   * investigate the **authenticator_state** for `authenticated: true`","ref":"ExFTP.Auth.BearerAuth.html#authenticated?/1-workflow"},{"type":"function","title":"⤵️ Returns - ExFTP.Auth.BearerAuth.authenticated?/1","doc":"**✅ On Success**\n\n  ```elixir\n  `true` or `false`\n  ```","ref":"ExFTP.Auth.BearerAuth.html#authenticated?/1-returns"},{"type":"function","title":"💻 Examples - ExFTP.Auth.BearerAuth.authenticated?/1","doc":"iex> alias ExFTP.Auth.BearerAuth\n    iex> Application.put_env(:ex_ftp, :authenticator, ExFTP.Auth.BearerAuth)\n    iex> Application.put_env(:ex_ftp, :authenticator_config, %{\n    iex>  login_url: \"https://httpbin.dev/bearer\",\n    iex>  authenticated_url: \"https://httpbin.dev/bearer\",\n    iex>  authenticated_method: :get,\n    iex> })\n    iex> BearerAuth.authenticated?(%{bearer_token: \"my.bearer.token\"})\n    true","ref":"ExFTP.Auth.BearerAuth.html#authenticated?/1-examples"},{"type":"function","title":"👀 See Also - ExFTP.Auth.BearerAuth.authenticated?/1","doc":"* `t:ExFTP.Auth.BearerAuthConfig.t/0`\n  * `t:ExFTP.Auth.Common.authenticated_url/0`\n  * `t:ExFTP.Auth.Common.authenticated_method/0`\n\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.BearerAuth.html#authenticated?/1-see-also"},{"type":"function","title":"ExFTP.Auth.BearerAuth.login/2","doc":"Requests a login using a\n[Bearer Token](https://swagger.io/docs/specification/v3_0/authentication/bearer-authentication/){:target=\"_blank\"}\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.BearerAuth.html#login/2"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.BearerAuth.login/2","doc":"* **password** :: `t:ExFTP.Authenticator.password/0`\n  * **authenticator_state** :: `t:ExFTP.Authenticator.authenticator_state/0`","ref":"ExFTP.Auth.BearerAuth.html#login/2-params"},{"type":"function","title":"🧑‍🍳 Workflow - ExFTP.Auth.BearerAuth.login/2","doc":"* Reads the **authenticator_config**.\n * Receives a bearer token from the client (as a password)\n * Calls the **login_url** with the proper bearer token headers\n * If the response is **HTTP 200**, success. Otherwise, bad login.","ref":"ExFTP.Auth.BearerAuth.html#login/2-workflow"},{"type":"function","title":"⤵️ Returns - ExFTP.Auth.BearerAuth.login/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, authenticator_state}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, bad_login}\n  ```","ref":"ExFTP.Auth.BearerAuth.html#login/2-returns"},{"type":"function","title":"💻 Examples - ExFTP.Auth.BearerAuth.login/2","doc":"iex> alias ExFTP.Auth.BearerAuth\n    iex> Application.put_env(:ex_ftp, :authenticator, ExFTP.Auth.BearerAuth)\n    iex> Application.put_env(:ex_ftp, :authenticator_config, %{\n    iex>  login_url: \"https://httpbin.dev/bearer\",\n    iex>  login_method: :post\n    iex> })\n    iex> {:ok, _} = BearerAuth.login(\"my.bearer.token\" , %{})","ref":"ExFTP.Auth.BearerAuth.html#login/2-examples"},{"type":"function","title":"👀 See Also - ExFTP.Auth.BearerAuth.login/2","doc":"* `t:ExFTP.Auth.BearerAuthConfig.t/0`\n  * `t:ExFTP.Auth.Common.login_url/0`\n  * `t:ExFTP.Auth.Common.login_method/0`\n\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.BearerAuth.html#login/2-see-also"},{"type":"function","title":"ExFTP.Auth.BearerAuth.valid_user?/1","doc":"Always returns `true`.\n\n> #### No performance benefit {: .tip}\n> This method is normally used to short-circuit bad login requests.\n> The performance gain in that short-circuit is negligible for this authenticator, so it's not used.","ref":"ExFTP.Auth.BearerAuth.html#valid_user?/1"},{"type":"module","title":"ExFTP.Auth.DigestAuth","doc":"When **authenticator** is `ExFTP.Auth.DigestAuth`, this authenticator will call out to an HTTP endpoint that\nimplements [HTTP Digest Access Auth](https://en.wikipedia.org/wiki/Digest_access_authentication){:target=\"_blank\"}\nwith the user's supplied credentials.\n\n> #### 🔒 Security {: .tip}\n>\n> This can be used in situations where SSL is not available, though be warned, Digest Access is considered\n> an obsolete protocol.\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.DigestAuth.html"},{"type":"module","title":"⚙️ Configuration - ExFTP.Auth.DigestAuth","doc":"*Keys*\n\n* **authenticator**  == `ExFTP.Auth.DigestAuth`\n* **authenticator_config** :: `t:ExFTP.Auth.DigestAuthConfig.t/0`\n\n*Example*\n\n```elixir\n  %{\n    # ... ,\n    authenticator: ExFTP.Auth.DigestAuth,\n    authenticator_config: %{\n      login_url: \"https://httpbin.dev/digest-auth/auth/replace/me/MD5\",\n      login_method: :get,\n      authenticated_url: \"https://httpbin.dev/digest-auth/auth/replace/me/MD5\",\n      authenticated_method: :get,\n      authenticated_ttl_ms: 1000 * 60 * 60\n    }\n  }\n```","ref":"ExFTP.Auth.DigestAuth.html#module-configuration"},{"type":"module","title":"👀 See Also - ExFTP.Auth.DigestAuth","doc":"* `ExFTP.Authenticator`","ref":"ExFTP.Auth.DigestAuth.html#module-see-also"},{"type":"module","title":"📖 Resources - ExFTP.Auth.DigestAuth","doc":"* 📖 [RFC 959 (section-4)](https://www.rfc-editor.org/rfc/rfc959#section-4){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Auth.DigestAuth.html#module-resources"},{"type":"function","title":"ExFTP.Auth.DigestAuth.authenticated?/1","doc":"Determines whether this session is still considered authenticated\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.DigestAuth.html#authenticated?/1"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.DigestAuth.authenticated?/1","doc":"* **authenticator_state** :: `t:ExFTP.Authenticator.authenticator_state/0`","ref":"ExFTP.Auth.DigestAuth.html#authenticated?/1-params"},{"type":"function","title":"🧑‍🍳 Workflow - ExFTP.Auth.DigestAuth.authenticated?/1","doc":"* Reads the **authenticator_config**.\n * If the config has **authenticated_url**,\n    * Calls the **authenticated_url** - receives **HTTP 401** with digest headers\n    * Performs calculation, calls **authenticated_url** with proper headers\n    * If the response is **HTTP 200**, success. Otherwise, bad login.\n * If the config does not have **authenticated_url**,\n   * investigate the **authenticator_state** for `authenticated: true`","ref":"ExFTP.Auth.DigestAuth.html#authenticated?/1-workflow"},{"type":"function","title":"⤵️ Returns - ExFTP.Auth.DigestAuth.authenticated?/1","doc":"**✅ On Success**\n\n  ```elixir\n  `true` or `false`\n  ```","ref":"ExFTP.Auth.DigestAuth.html#authenticated?/1-returns"},{"type":"function","title":"💻 Examples - ExFTP.Auth.DigestAuth.authenticated?/1","doc":"iex> alias ExFTP.Auth.DigestAuth\n    iex> username = \"alice\"\n    iex> password = \"password1234\"\n    iex> Application.put_env(:ex_ftp, :authenticator, ExFTP.Auth.DigestAuth)\n    iex> Application.put_env(:ex_ftp, :authenticator_config, %{\n    iex>  login_url: \"https://httpbin.dev/digest-auth/auth/\" <> username <> \"/\" <> password <> \"/MD5\",\n    iex>  authenticated_url: \"https://httpbin.dev/digest-auth/auth/\" <> username <> \"/\" <> password <> \"/MD5\",\n    iex>  authenticated_method: :get,\n    iex> })\n    iex> DigestAuth.authenticated?(%{username: username, password: password})\n    true","ref":"ExFTP.Auth.DigestAuth.html#authenticated?/1-examples"},{"type":"function","title":"👀 See Also - ExFTP.Auth.DigestAuth.authenticated?/1","doc":"* `t:ExFTP.Auth.DigestAuthConfig.t/0`\n  * `t:ExFTP.Auth.Common.authenticated_url/0`\n  * `t:ExFTP.Auth.Common.authenticated_method/0`\n\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.DigestAuth.html#authenticated?/1-see-also"},{"type":"function","title":"ExFTP.Auth.DigestAuth.login/2","doc":"Requests a login using [HTTP Digest Access Auth](https://en.wikipedia.org/wiki/Digest_access_authentication){:target=\"_blank\"}\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.DigestAuth.html#login/2"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.DigestAuth.login/2","doc":"* **password** :: `t:ExFTP.Authenticator.password/0`\n  * **authenticator_state** :: `t:ExFTP.Authenticator.authenticator_state/0`","ref":"ExFTP.Auth.DigestAuth.html#login/2-params"},{"type":"function","title":"🧑‍🍳 Workflow - ExFTP.Auth.DigestAuth.login/2","doc":"* Reads the **authenticator_config**.\n * Receives a password from the client (a username was supplied earlier)\n * Calls the **login_url** - receives **HTTP 401** with digest headers\n * Performs calculation, calls **login_url** with proper headers\n * If the response is **HTTP 200**, success. Otherwise, bad login.","ref":"ExFTP.Auth.DigestAuth.html#login/2-workflow"},{"type":"function","title":"⤵️ Returns - ExFTP.Auth.DigestAuth.login/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, authenticator_state}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, bad_login}\n  ```","ref":"ExFTP.Auth.DigestAuth.html#login/2-returns"},{"type":"function","title":"💻 Examples - ExFTP.Auth.DigestAuth.login/2","doc":"iex> alias ExFTP.Auth.DigestAuth\n    iex> username = \"alice\"\n    iex> password = \"password1234\"\n    iex> Application.put_env(:ex_ftp, :authenticator, ExFTP.Auth.DigestAuth)\n    iex> Application.put_env(:ex_ftp, :authenticator_config, %{\n    iex>  login_url: \"https://httpbin.dev/digest-auth/auth/\" <> username <> \"/\" <> password <> \"/MD5\",\n    iex>  login_method: :get\n    iex> })\n    iex> {:ok, _} = DigestAuth.login(password , %{username: username})","ref":"ExFTP.Auth.DigestAuth.html#login/2-examples"},{"type":"function","title":"👀 See Also - ExFTP.Auth.DigestAuth.login/2","doc":"* `t:ExFTP.Auth.DigestAuthConfig.t/0`\n  * `t:ExFTP.Auth.Common.login_url/0`\n  * `t:ExFTP.Auth.Common.login_method/0`\n\n\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.DigestAuth.html#login/2-see-also"},{"type":"function","title":"ExFTP.Auth.DigestAuth.valid_user?/1","doc":"Always returns `true`.\n\n> #### No performance benefit {: .tip}\n> This method is normally used to short-circuit bad login requests.\n> The performance gain in that short-circuit is negligible for this auth, so it's not used.","ref":"ExFTP.Auth.DigestAuth.html#valid_user?/1"},{"type":"module","title":"ExFTP.Auth.NoAuth","doc":"When **authenticator** is `ExFTP.Auth.NoAuth`, this authenticator will completely ignore any supplied credentials and\nassume everything is authenticated.\n\n> #### 🔒 Security {: .error}\n>\n> Don't use `NoAuth` for production servers.\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.NoAuth.html"},{"type":"module","title":"⚙️ Configuration - ExFTP.Auth.NoAuth","doc":"*Keys*\n\n* **authenticator**  == `ExFTP.Auth.NoAuth`\n* **authenticator_config** == `%{}`\n\n*Example*\n\n```elixir\n  %{\n    # ... ,\n    authenticator: ExFTP.Auth.NoAuth,\n    authenticator_config: %{}\n  }\n```","ref":"ExFTP.Auth.NoAuth.html#module-configuration"},{"type":"module","title":"👀 See Also - ExFTP.Auth.NoAuth","doc":"* `ExFTP.Authenticator`","ref":"ExFTP.Auth.NoAuth.html#module-see-also"},{"type":"module","title":"📖 Resources - ExFTP.Auth.NoAuth","doc":"* 📖 [RFC 959 (section-4)](https://www.rfc-editor.org/rfc/rfc959#section-4){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Auth.NoAuth.html#module-resources"},{"type":"function","title":"ExFTP.Auth.NoAuth.authenticated?/1","doc":"Assumes the user is always authenticated\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.NoAuth.html#authenticated?/1"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.NoAuth.authenticated?/1","doc":"* **authenticator_state** :: `t:ExFTP.Authenticator.authenticator_state/0`","ref":"ExFTP.Auth.NoAuth.html#authenticated?/1-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Auth.NoAuth.authenticated?/1","doc":"**✅ On Success**\n\n  ```elixir\n  `true`\n  ```","ref":"ExFTP.Auth.NoAuth.html#authenticated?/1-returns"},{"type":"function","title":"💻 Examples - ExFTP.Auth.NoAuth.authenticated?/1","doc":"iex> alias ExFTP.Auth.NoAuth\n    iex> NoAuth.authenticated?(%{authenticated: true})\n    true\n    iex> NoAuth.authenticated?(%{})\n    true","ref":"ExFTP.Auth.NoAuth.html#authenticated?/1-examples"},{"type":"function","title":"📖 Resources - ExFTP.Auth.NoAuth.authenticated?/1","doc":"* 📖 [RFC 959 (section-4)](https://www.rfc-editor.org/rfc/rfc959#section-4){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Auth.NoAuth.html#authenticated?/1-resources"},{"type":"function","title":"ExFTP.Auth.NoAuth.login/2","doc":"Login will always succeed.\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.NoAuth.html#login/2"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.NoAuth.login/2","doc":"* **password** :: `t:ExFTP.Authenticator.password/0`\n  * **authenticator_state** :: `t:ExFTP.Authenticator.authenticator_state/0`","ref":"ExFTP.Auth.NoAuth.html#login/2-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Auth.NoAuth.login/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, authenticator_state}\n  ```","ref":"ExFTP.Auth.NoAuth.html#login/2-returns"},{"type":"function","title":"💻 Examples - ExFTP.Auth.NoAuth.login/2","doc":"iex> alias ExFTP.Auth.NoAuth\n    iex> {:ok, _auth_state} = NoAuth.login(\"password\", %{username: \"jsmith\"})\n    iex> {:ok, _} = NoAuth.login(\"password\", %{})\n    iex> {:ok, _} = NoAuth.login(\"password\", %{username: \"root\"})\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.NoAuth.html#login/2-examples"},{"type":"function","title":"ExFTP.Auth.NoAuth.valid_user?/1","doc":"Always returns true\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.NoAuth.html#valid_user?/1"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.NoAuth.valid_user?/1","doc":"* **username** :: `t:ExFTP.Authenticator.username/0`","ref":"ExFTP.Auth.NoAuth.html#valid_user?/1-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Auth.NoAuth.valid_user?/1","doc":"**✅ On Success**\n\n  ```elixir\n  `true`\n  ```","ref":"ExFTP.Auth.NoAuth.html#valid_user?/1-returns"},{"type":"function","title":"💻 Examples - ExFTP.Auth.NoAuth.valid_user?/1","doc":"iex> alias ExFTP.Auth.NoAuth\n    iex> NoAuth.valid_user?(\"jsmith\")\n    true\n    iex> NoAuth.valid_user?(\"root\")\n    true\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.NoAuth.html#valid_user?/1-examples"},{"type":"module","title":"ExFTP.Auth.PassthroughAuth","doc":"When **authenticator** is `ExFTP.Auth.PassthroughAuth`, this authenticator will require credentials,\nbut accepts any user and password combination who isn't `root`.\n\n> #### 🔒 Security {: .error}\n>\n> Don't use `PassthroughAuth` for production servers.\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.PassthroughAuth.html"},{"type":"module","title":"⚙️ Configuration - ExFTP.Auth.PassthroughAuth","doc":"*Keys*\n\n* **authenticator**  == `ExFTP.Auth.PassthroughAuth`\n* **authenticator_config** == `%{}`\n\n*Example*\n\n```elixir\n  %{\n    # ... ,\n    authenticator: ExFTP.Auth.PassthroughAuth,\n    authenticator_config: %{}\n  }\n```","ref":"ExFTP.Auth.PassthroughAuth.html#module-configuration"},{"type":"module","title":"👀 See Also - ExFTP.Auth.PassthroughAuth","doc":"* `ExFTP.Authenticator`","ref":"ExFTP.Auth.PassthroughAuth.html#module-see-also"},{"type":"module","title":"📖 Resources - ExFTP.Auth.PassthroughAuth","doc":"* 📖 [RFC 959 (section-4)](https://www.rfc-editor.org/rfc/rfc959#section-4){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Auth.PassthroughAuth.html#module-resources"},{"type":"function","title":"ExFTP.Auth.PassthroughAuth.authenticated?/1","doc":"Assumes the user is still authenticated as long as `authenticated: true`\n  still exists in the **authenticator_state**.\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.PassthroughAuth.html#authenticated?/1"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.PassthroughAuth.authenticated?/1","doc":"* **authenticator_state** :: `t:ExFTP.Authenticator.authenticator_state/0`","ref":"ExFTP.Auth.PassthroughAuth.html#authenticated?/1-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Auth.PassthroughAuth.authenticated?/1","doc":"**✅ On Success**\n\n  ```elixir\n  `true` or `false`\n  ```","ref":"ExFTP.Auth.PassthroughAuth.html#authenticated?/1-returns"},{"type":"function","title":"💻 Examples - ExFTP.Auth.PassthroughAuth.authenticated?/1","doc":"iex> alias ExFTP.Auth.PassthroughAuth\n    iex> PassthroughAuth.authenticated?(%{authenticated: true})\n    true\n    iex> PassthroughAuth.authenticated?(%{})\n    false\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.PassthroughAuth.html#authenticated?/1-examples"},{"type":"function","title":"ExFTP.Auth.PassthroughAuth.login/2","doc":"Login will respond `{:ok, unmodified_auth_state}` to anyone but `username: \"root\"`\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.PassthroughAuth.html#login/2"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.PassthroughAuth.login/2","doc":"* **password** :: `t:ExFTP.Authenticator.password/0`\n  * **authenticator_state** :: `t:ExFTP.Authenticator.authenticator_state/0`","ref":"ExFTP.Auth.PassthroughAuth.html#login/2-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Auth.PassthroughAuth.login/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, authenticator_state}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, bad_login}\n  ```","ref":"ExFTP.Auth.PassthroughAuth.html#login/2-returns"},{"type":"function","title":"💻 Examples - ExFTP.Auth.PassthroughAuth.login/2","doc":"iex> alias ExFTP.Auth.PassthroughAuth\n    iex> {:ok, _auth_state} = PassthroughAuth.login(\"password\", %{username: \"jsmith\"})\n    iex> {:error, _} = PassthroughAuth.login(\"password\", %{})\n    iex> # \"root\" is a disallowed user in PassthroughAuth\n    iex> {:error, _} = PassthroughAuth.login(\"password\", %{username: \"root\"})\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.PassthroughAuth.html#login/2-examples"},{"type":"function","title":"ExFTP.Auth.PassthroughAuth.valid_user?/1","doc":"Returns `true` if **username** is anything except `\"root\"`\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.PassthroughAuth.html#valid_user?/1"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.PassthroughAuth.valid_user?/1","doc":"* **username** :: `t:ExFTP.Authenticator.username/0`","ref":"ExFTP.Auth.PassthroughAuth.html#valid_user?/1-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Auth.PassthroughAuth.valid_user?/1","doc":"**✅ On Success**\n\n  ```elixir\n  `true` or `false`\n  ```","ref":"ExFTP.Auth.PassthroughAuth.html#valid_user?/1-returns"},{"type":"function","title":"💻 Examples - ExFTP.Auth.PassthroughAuth.valid_user?/1","doc":"iex> alias ExFTP.Auth.PassthroughAuth\n    iex> PassthroughAuth.valid_user?(\"jsmith\")\n    true\n    iex> PassthroughAuth.valid_user?(\"root\")\n    false","ref":"ExFTP.Auth.PassthroughAuth.html#valid_user?/1-examples"},{"type":"function","title":"⚠️ Reminders - ExFTP.Auth.PassthroughAuth.valid_user?/1","doc":"> #### 🔒 Security {: .tip}\n>\n> The client will never be informed that a username is invalid.\n>\n> The server uses this method to short-circuit bad auth calls.\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.PassthroughAuth.html#valid_user?/1-reminders"},{"type":"module","title":"ExFTP.Auth.WebhookAuth","doc":"When **authenticator** is `ExFTP.Auth.WebhookAuth`, this authenticator will call out to an HTTP endpoint that accepts\ntwo query parameters: `username` and/or `password_hash`.\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.WebhookAuth.html"},{"type":"module","title":"⚙️ Configuration - ExFTP.Auth.WebhookAuth","doc":"*Keys*\n\n* **authenticator**  == `ExFTP.Auth.WebhookAuth`\n* **authenticator_config** :: `t:ExFTP.Auth.WebhookAuthConfig.t/0`\n\n*Example*\n\n```elixir\n  %{\n    # ... ,\n    authenticator: ExFTP.Auth.WebhookAuth,\n    authenticator_config: %{\n      login_url: \"https://httpbin.dev/status/200\",\n      login_method: :post,\n      password_hash_type: :sha256,\n      authenticated_url: \"https://httpbin.dev/status/200\",\n      authenticated_method: :post,\n      authenticated_ttl_ms: 1000 * 60 * 60\n    }\n  }\n```","ref":"ExFTP.Auth.WebhookAuth.html#module-configuration"},{"type":"module","title":"👀 See Also - ExFTP.Auth.WebhookAuth","doc":"* `ExFTP.Authenticator`","ref":"ExFTP.Auth.WebhookAuth.html#module-see-also"},{"type":"module","title":"📖 Resources - ExFTP.Auth.WebhookAuth","doc":"* 📖 [RFC 959 (section-4)](https://www.rfc-editor.org/rfc/rfc959#section-4){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Auth.WebhookAuth.html#module-resources"},{"type":"function","title":"ExFTP.Auth.WebhookAuth.authenticated?/1","doc":"Determines whether this session is still considered authenticated\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.WebhookAuth.html#authenticated?/1"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.WebhookAuth.authenticated?/1","doc":"* **authenticator_state** :: `t:ExFTP.Authenticator.authenticator_state/0`","ref":"ExFTP.Auth.WebhookAuth.html#authenticated?/1-params"},{"type":"function","title":"🧑‍🍳 Workflow - ExFTP.Auth.WebhookAuth.authenticated?/1","doc":"* Reads the **authenticator_config**.\n * If the config has **authenticated_url**,\n   * Calls it with the username(e.g `http://httpbin.dev/get?username={username}`)\n   * If the response is **HTTP 200**, success. Otherwise, no longer authenticated.\n * If the config does not have **authenticated_url**,\n   * investigate the **authenticator_state** for `authenticated: true`","ref":"ExFTP.Auth.WebhookAuth.html#authenticated?/1-workflow"},{"type":"function","title":"⤵️ Returns - ExFTP.Auth.WebhookAuth.authenticated?/1","doc":"**✅ On Success**\n\n  ```elixir\n  `true` or `false`\n  ```","ref":"ExFTP.Auth.WebhookAuth.html#authenticated?/1-returns"},{"type":"function","title":"💻 Examples - ExFTP.Auth.WebhookAuth.authenticated?/1","doc":"iex> alias ExFTP.Auth.WebhookAuth\n    iex> Application.put_env(:ex_ftp, :authenticator, ExFTP.Auth.WebhookAuth)\n    iex> Application.put_env(:ex_ftp, :authenticator_config, %{\n    iex>  login_url: \"https://httpbin.dev/status/200\",\n    iex>  authenticated_url: \"https://httpbin.dev/get\",\n    iex>  authenticated_method: :get,\n    iex> })\n    iex> WebhookAuth.authenticated?(%{username: \"jsmith\"})\n    true","ref":"ExFTP.Auth.WebhookAuth.html#authenticated?/1-examples"},{"type":"function","title":"👀 See Also - ExFTP.Auth.WebhookAuth.authenticated?/1","doc":"* `t:ExFTP.Auth.WebhookAuthConfig.t/0`\n  * `t:ExFTP.Auth.Common.authenticated_url/0`\n  * `t:ExFTP.Auth.Common.authenticated_method/0`\n\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.WebhookAuth.html#authenticated?/1-see-also"},{"type":"function","title":"ExFTP.Auth.WebhookAuth.login/2","doc":"Requests a login using a webhook.\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.WebhookAuth.html#login/2"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.WebhookAuth.login/2","doc":"* **password** :: `t:ExFTP.Authenticator.password/0`\n  * **authenticator_state** :: `t:ExFTP.Authenticator.authenticator_state/0`","ref":"ExFTP.Auth.WebhookAuth.html#login/2-params"},{"type":"function","title":"🧑‍🍳 Workflow - ExFTP.Auth.WebhookAuth.login/2","doc":"* Reads the **authenticator_config**.\n * Receives a password from the client (a `:username` key might exist in the **authenticator_state**)\n * Hashes the password\n * Calls the **login_url** (e.g `http://httpbin.dev/get?username={username}&password_hash={password_hash}`)\n * If the response is **HTTP 200**, success. Otherwise, bad login.","ref":"ExFTP.Auth.WebhookAuth.html#login/2-workflow"},{"type":"function","title":"⤵️ Returns - ExFTP.Auth.WebhookAuth.login/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, authenticator_state}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, bad_login}\n  ```","ref":"ExFTP.Auth.WebhookAuth.html#login/2-returns"},{"type":"function","title":"💻 Examples - ExFTP.Auth.WebhookAuth.login/2","doc":"iex> alias ExFTP.Auth.WebhookAuth\n    iex> Application.put_env(:ex_ftp, :authenticator, ExFTP.Auth.WebhookAuth)\n    iex> Application.put_env(:ex_ftp, :authenticator_config, %{\n    iex>  login_url: \"https://httpbin.dev/status/200\",\n    iex>  login_method: :post,\n    iex>  password_hash_type: :sha256\n    iex> })\n    iex> {:ok, _} = WebhookAuth.login(\"password123\", %{username: \"jsmith\"})","ref":"ExFTP.Auth.WebhookAuth.html#login/2-examples"},{"type":"function","title":"👀 See Also - ExFTP.Auth.WebhookAuth.login/2","doc":"* `t:ExFTP.Auth.WebhookAuthConfig.t/0`\n  * `t:ExFTP.Auth.Common.login_url/0`\n  * `t:ExFTP.Auth.Common.login_method/0`\n  * `t:ExFTP.Auth.WebhookAuthConfig.password_hash_type/0`\n\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.WebhookAuth.html#login/2-see-also"},{"type":"function","title":"ExFTP.Auth.WebhookAuth.valid_user?/1","doc":"Always returns `true`.\n\n> #### No performance benefit {: .tip}\n> This method is normally used to short-circuit bad login requests.\n> The performance gain in that short-circuit is negligible for webhooks, so it's not used.","ref":"ExFTP.Auth.WebhookAuth.html#valid_user?/1"},{"type":"behaviour","title":"ExFTP.Authenticator","doc":"A behaviour defining an Authenticator.\n\nAuthenticators are used by the FTP interface to verify login credentials.\n\n<!-- tabs-open -->","ref":"ExFTP.Authenticator.html"},{"type":"behaviour","title":"👀 See Also - ExFTP.Authenticator","doc":"* `ExFTP.Auth.WebhookAuth`\n  * `ExFTP.Auth.PassthroughAuth`\n  * `ExFTP.Auth.NoAuth`","ref":"ExFTP.Authenticator.html#module-see-also"},{"type":"behaviour","title":"📖 Resources - ExFTP.Authenticator","doc":"* 📖 [RFC 959 (section-4)](https://www.rfc-editor.org/rfc/rfc959#section-4){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Authenticator.html#module-resources"},{"type":"callback","title":"ExFTP.Authenticator.authenticated?/1","doc":"Given the current state, whether this session is still considered authenticated\n\n<!-- tabs-open -->","ref":"ExFTP.Authenticator.html#c:authenticated?/1"},{"type":"callback","title":"🏷️ Params - ExFTP.Authenticator.authenticated?/1","doc":"* **authenticator_state** :: `t:authenticator_state/0`","ref":"ExFTP.Authenticator.html#c:authenticated?/1-params"},{"type":"callback","title":"⤵️ Returns - ExFTP.Authenticator.authenticated?/1","doc":"**✅ On Success**\n\n  ```elixir\n  `true` or `false`\n  ```","ref":"ExFTP.Authenticator.html#c:authenticated?/1-returns"},{"type":"callback","title":"💻 Examples - ExFTP.Authenticator.authenticated?/1","doc":"iex> alias ExFTP.Auth.PassthroughAuth\n    iex> PassthroughAuth.authenticated?(%{authenticated: true})\n    true\n    iex> PassthroughAuth.authenticated?(%{})\n    false","ref":"ExFTP.Authenticator.html#c:authenticated?/1-examples"},{"type":"callback","title":"⚠️ Reminders - ExFTP.Authenticator.authenticated?/1","doc":"> #### Authenticator State {: .tip}\n>\n> The `t:authenticator_state/0` will contain `authenticated: true`\n> if login has succeeded before in this session.\n>\n> Authenticators may choose to drop that key for their own use cases (e.g if a TTL expires)","ref":"ExFTP.Authenticator.html#c:authenticated?/1-reminders"},{"type":"callback","title":"📖 Resources - ExFTP.Authenticator.authenticated?/1","doc":"* 📖 [RFC 959 (section-4)](https://www.rfc-editor.org/rfc/rfc959#section-4){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Authenticator.html#c:authenticated?/1-resources"},{"type":"type","title":"ExFTP.Authenticator.authenticator_state/0","doc":"State held onto by the server and modified by the `Authenticator`.\n\nIt may be used by the authenticator to keep stateful values.\n\n<!-- tabs-open -->","ref":"ExFTP.Authenticator.html#t:authenticator_state/0"},{"type":"type","title":"⚠️ Reminders - ExFTP.Authenticator.authenticator_state/0","doc":"> #### Special Keys {: .tip}\n>\n>   * `authenticated:` `true` will exist if the current user has successfully called `c:login/2`\n>      during this session (and the Authenticator hasn't otherwise removed it)\n>   * `username:` `t:username/0` will exist if the current session has defined the user\n>      (but hasn't necessarily supplied a password)","ref":"ExFTP.Authenticator.html#t:authenticator_state/0-reminders"},{"type":"type","title":"👀 See Also - ExFTP.Authenticator.authenticator_state/0","doc":"* `c:login/2`\n  * `c:authenticated?/1`","ref":"ExFTP.Authenticator.html#t:authenticator_state/0-see-also"},{"type":"type","title":"📖 Resources - ExFTP.Authenticator.authenticator_state/0","doc":"* 📖 [RFC 959 (section-4)](https://www.rfc-editor.org/rfc/rfc959#section-4){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n<!-- tabs-close -->","ref":"ExFTP.Authenticator.html#t:authenticator_state/0-resources"},{"type":"callback","title":"ExFTP.Authenticator.login/2","doc":"Request a login given a security challenge value.\n\n<!-- tabs-open -->","ref":"ExFTP.Authenticator.html#c:login/2"},{"type":"callback","title":"🏷️ Params - ExFTP.Authenticator.login/2","doc":"* **password** :: `t:password/0`\n  * **authenticator_state** :: `t:authenticator_state/0`","ref":"ExFTP.Authenticator.html#c:login/2-params"},{"type":"callback","title":"⤵️ Returns - ExFTP.Authenticator.login/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, authenticator_state}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, bad_login}\n  ```","ref":"ExFTP.Authenticator.html#c:login/2-returns"},{"type":"callback","title":"💻 Examples - ExFTP.Authenticator.login/2","doc":"iex> alias ExFTP.Auth.PassthroughAuth\n    iex> {:ok, _auth_state} = PassthroughAuth.login(\"password\", %{username: \"jsmith\"})\n    iex> {:error, _} = PassthroughAuth.login(\"password\", %{})\n    iex> # \"root\" is a disallowed user in PassthroughAuth\n    iex> {:error, _} = PassthroughAuth.login(\"password\", %{username: \"root\"})","ref":"ExFTP.Authenticator.html#c:login/2-examples"},{"type":"callback","title":"⚠️ Reminders - ExFTP.Authenticator.login/2","doc":"> #### Authenticator State {: .tip}\n>\n> The `t:authenticator_state/0` will contain a `:username` key, if one was provided.\n>\n> On success, the **authenticator_state** will be automatically updated to include `authenticated: true`.\n> See `c:authenticated?/1` for more information.","ref":"ExFTP.Authenticator.html#c:login/2-reminders"},{"type":"callback","title":"📖 Resources - ExFTP.Authenticator.login/2","doc":"* 📖 [RFC 959 (section-4)](https://www.rfc-editor.org/rfc/rfc959#section-4){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Authenticator.html#c:login/2-resources"},{"type":"type","title":"ExFTP.Authenticator.password/0","doc":"The string value of the security challenge.\n\n<!-- tabs-open -->","ref":"ExFTP.Authenticator.html#t:password/0"},{"type":"type","title":"⚠️ Reminders - ExFTP.Authenticator.password/0","doc":"> #### What is a password, really? {: .tip}\n>\n> A password could be used in the traditional sense; but this could also be a OTP, a hash,\n> or any string value that the user would use to authenticate","ref":"ExFTP.Authenticator.html#t:password/0-reminders"},{"type":"type","title":"👀 See Also - ExFTP.Authenticator.password/0","doc":"* `c:login/2`","ref":"ExFTP.Authenticator.html#t:password/0-see-also"},{"type":"type","title":"📖 Resources - ExFTP.Authenticator.password/0","doc":"* 📖 [RFC 959 (section-4)](https://www.rfc-editor.org/rfc/rfc959#section-4){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Authenticator.html#t:password/0-resources"},{"type":"type","title":"ExFTP.Authenticator.username/0","doc":"A string representing a user (e.g `\"jsmith\"`)\n\n<!-- tabs-open -->","ref":"ExFTP.Authenticator.html#t:username/0"},{"type":"type","title":"👀 See Also - ExFTP.Authenticator.username/0","doc":"* `c:valid_user?/1`","ref":"ExFTP.Authenticator.html#t:username/0-see-also"},{"type":"type","title":"📖 Resources - ExFTP.Authenticator.username/0","doc":"* 📖 [RFC 959 (section-4)](https://www.rfc-editor.org/rfc/rfc959#section-4){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Authenticator.html#t:username/0-resources"},{"type":"callback","title":"ExFTP.Authenticator.valid_user?/1","doc":"Whether a given username is valid.\n\n<!-- tabs-open -->","ref":"ExFTP.Authenticator.html#c:valid_user?/1"},{"type":"callback","title":"🏷️ Params - ExFTP.Authenticator.valid_user?/1","doc":"* **username** :: `t:username/0`","ref":"ExFTP.Authenticator.html#c:valid_user?/1-params"},{"type":"callback","title":"⤵️ Returns - ExFTP.Authenticator.valid_user?/1","doc":"**✅ On Success**\n\n  ```elixir\n  `true` or `false`\n  ```","ref":"ExFTP.Authenticator.html#c:valid_user?/1-returns"},{"type":"callback","title":"💻 Examples - ExFTP.Authenticator.valid_user?/1","doc":"iex> alias ExFTP.Auth.PassthroughAuth\n    iex> PassthroughAuth.valid_user?(\"jsmith\")\n    true\n    iex> PassthroughAuth.valid_user?(\"root\")\n    false","ref":"ExFTP.Authenticator.html#c:valid_user?/1-examples"},{"type":"callback","title":"⚠️ Reminders - ExFTP.Authenticator.valid_user?/1","doc":"> #### 🔒 Security {: .tip}\n>\n> The client will never be informed that a username is invalid.\n>\n> The server uses this method to short-circuit auth calls.","ref":"ExFTP.Authenticator.html#c:valid_user?/1-reminders"},{"type":"callback","title":"📖 Resources - ExFTP.Authenticator.valid_user?/1","doc":"* 📖 [RFC 959 (section-4)](https://www.rfc-editor.org/rfc/rfc959#section-4){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Authenticator.html#c:valid_user?/1-resources"},{"type":"module","title":"ExFTP.Auth.BasicAuthConfig","doc":"A module describing the **authenticator_config** value for `ExFTP.Auth.BasicAuth`\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.BasicAuthConfig.html"},{"type":"module","title":"👀 See Also - ExFTP.Auth.BasicAuthConfig","doc":"* `ExFTP.Auth.BasicAuth`\n  * `ExFTP.Authenticator`","ref":"ExFTP.Auth.BasicAuthConfig.html#module-see-also"},{"type":"module","title":"📖 Resources - ExFTP.Auth.BasicAuthConfig","doc":"* 📖 [RFC 959](https://www.rfc-editor.org/rfc/rfc959){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Auth.BasicAuthConfig.html#module-resources"},{"type":"function","title":"ExFTP.Auth.BasicAuthConfig.build/1","doc":"Builds a `t:ExFTP.Auth.BasicAuthConfig.t/0` from a map\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.BasicAuthConfig.html#build/1"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.BasicAuthConfig.build/1","doc":"* **m** :: A map to build into a `t:ExFTP.Auth.BasicAuthConfig.t/0`\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.BasicAuthConfig.html#build/1-params"},{"type":"type","title":"ExFTP.Auth.BasicAuthConfig.t/0","doc":"The **authenticator_config** value for `ExFTP.Auth.BasicAuth`\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.BasicAuthConfig.html#t:t/0"},{"type":"type","title":"🏷️ Required Keys - ExFTP.Auth.BasicAuthConfig.t/0","doc":"* **login_url** :: `t:ExFTP.Auth.Common.login_url/0`","ref":"ExFTP.Auth.BasicAuthConfig.html#t:t/0-required-keys"},{"type":"type","title":"🏷️ Optional Keys - ExFTP.Auth.BasicAuthConfig.t/0","doc":"* **login_method** :: `t:ExFTP.Auth.Common.login_method/0`\n  * **authenticated_url** :: `t:ExFTP.Auth.Common.authenticated_url/0`\n  * **authenticated_method** :: `t:ExFTP.Auth.Common.authenticated_url/0`\n  * **authenticated_ttl_ms** :: `t:ExFTP.Auth.Common.authenticated_ttl_ms/0`\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.BasicAuthConfig.html#t:t/0-optional-keys"},{"type":"module","title":"ExFTP.Auth.BearerAuthConfig","doc":"A module describing the **authenticator_config** value for `ExFTP.Auth.BearerAuth`\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.BearerAuthConfig.html"},{"type":"module","title":"👀 See Also - ExFTP.Auth.BearerAuthConfig","doc":"* `ExFTP.Auth.BearerAuth`\n  * `ExFTP.Authenticator`","ref":"ExFTP.Auth.BearerAuthConfig.html#module-see-also"},{"type":"module","title":"📖 Resources - ExFTP.Auth.BearerAuthConfig","doc":"* 📖 [RFC 959](https://www.rfc-editor.org/rfc/rfc959){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Auth.BearerAuthConfig.html#module-resources"},{"type":"function","title":"ExFTP.Auth.BearerAuthConfig.build/1","doc":"Builds a `t:ExFTP.Auth.BearerAuthConfig.t/0` from a map\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.BearerAuthConfig.html#build/1"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.BearerAuthConfig.build/1","doc":"* **m** :: A map to build into a `t:ExFTP.Auth.BearerAuthConfig.t/0`\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.BearerAuthConfig.html#build/1-params"},{"type":"type","title":"ExFTP.Auth.BearerAuthConfig.t/0","doc":"The **authenticator_config** value for `ExFTP.Auth.BearerAuth`\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.BearerAuthConfig.html#t:t/0"},{"type":"type","title":"🏷️ Required Keys - ExFTP.Auth.BearerAuthConfig.t/0","doc":"* **login_url** :: `t:ExFTP.Auth.Common.login_url/0`","ref":"ExFTP.Auth.BearerAuthConfig.html#t:t/0-required-keys"},{"type":"type","title":"🏷️ Optional Keys - ExFTP.Auth.BearerAuthConfig.t/0","doc":"* **login_method** :: `t:ExFTP.Auth.Common.login_method/0`\n  * **authenticated_url** :: `t:ExFTP.Auth.Common.authenticated_url/0`\n  * **authenticated_method** :: `t:ExFTP.Auth.Common.authenticated_url/0`\n  * **authenticated_ttl_ms** :: `t:ExFTP.Auth.Common.authenticated_ttl_ms/0`\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.BearerAuthConfig.html#t:t/0-optional-keys"},{"type":"module","title":"ExFTP.Auth.Common","doc":"A module describing types functions common across all `ExFTP.Authenticator`","ref":"ExFTP.Auth.Common.html"},{"type":"type","title":"ExFTP.Auth.Common.authenticated_method/0","doc":"HTTP method paired with `t:authenticated_url/0`\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.Common.html#t:authenticated_method/0"},{"type":"type","title":"👀 See Also - ExFTP.Auth.Common.authenticated_method/0","doc":"* `c:ExFTP.Authenticator.authenticated?/1`\n\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.Common.html#t:authenticated_method/0-see-also"},{"type":"type","title":"ExFTP.Auth.Common.authenticated_ttl_ms/0","doc":"How many milliseconds pass before a user's session is not assumed still-authenticated.\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.Common.html#t:authenticated_ttl_ms/0"},{"type":"type","title":"👀 See Also - ExFTP.Auth.Common.authenticated_ttl_ms/0","doc":"* `c:ExFTP.Authenticator.authenticated?/1`\n\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.Common.html#t:authenticated_ttl_ms/0-see-also"},{"type":"type","title":"ExFTP.Auth.Common.authenticated_url/0","doc":"A URL used to check if a user should still be considered authenticated\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.Common.html#t:authenticated_url/0"},{"type":"type","title":"👀 See Also - ExFTP.Auth.Common.authenticated_url/0","doc":"* `c:ExFTP.Authenticator.authenticated?/1`\n\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.Common.html#t:authenticated_url/0-see-also"},{"type":"type","title":"ExFTP.Auth.Common.http_method/0","doc":"An HTTP method to use in a request.\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.Common.html#t:http_method/0"},{"type":"type","title":"👀 See Also - ExFTP.Auth.Common.http_method/0","doc":"* `c:ExFTP.Authenticator.authenticated?/1`\n  * `c:ExFTP.Authenticator.login/2`\n\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.Common.html#t:http_method/0-see-also"},{"type":"type","title":"ExFTP.Auth.Common.login_method/0","doc":"HTTP method paired with `t:login_url/0`\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.Common.html#t:login_method/0"},{"type":"type","title":"👀 See Also - ExFTP.Auth.Common.login_method/0","doc":"* `c:ExFTP.Authenticator.login/2`\n\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.Common.html#t:login_method/0-see-also"},{"type":"type","title":"ExFTP.Auth.Common.login_url/0","doc":"A URL used to log in a user\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.Common.html#t:login_url/0"},{"type":"type","title":"👀 See Also - ExFTP.Auth.Common.login_url/0","doc":"* `c:ExFTP.Authenticator.login/2`\n\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.Common.html#t:login_url/0-see-also"},{"type":"function","title":"ExFTP.Auth.Common.prepare/1","doc":"Cleans keys and values of a given map in preparation to build into a config\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.Common.html#prepare/1"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.Common.prepare/1","doc":"* **m** :: A map to prepare before building into a config\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.Common.html#prepare/1-params"},{"type":"function","title":"ExFTP.Auth.Common.validate_config/1","doc":"Grabs the `authenticator_config` from `Application.get_env/2` and validates it.\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.Common.html#validate_config/1"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.Common.validate_config/1","doc":"* **mod** :: A module to validate against (e.g `ExFTP.Auth.BasicAuthConfig`)","ref":"ExFTP.Auth.Common.html#validate_config/1-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Auth.Common.validate_config/1","doc":"**✅ On Success**\n\n  ```elixir\n  `{:ok, config}`\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  `{:error, desc}`\n  ```\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.Common.html#validate_config/1-returns"},{"type":"module","title":"ExFTP.Auth.DigestAuthConfig","doc":"A module describing the **authenticator_config** value for `ExFTP.Auth.DigestAuth`\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.DigestAuthConfig.html"},{"type":"module","title":"👀 See Also - ExFTP.Auth.DigestAuthConfig","doc":"* `ExFTP.Auth.DigestAuth`\n  * `ExFTP.Authenticator`","ref":"ExFTP.Auth.DigestAuthConfig.html#module-see-also"},{"type":"module","title":"📖 Resources - ExFTP.Auth.DigestAuthConfig","doc":"* 📖 [RFC 959](https://www.rfc-editor.org/rfc/rfc959){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Auth.DigestAuthConfig.html#module-resources"},{"type":"function","title":"ExFTP.Auth.DigestAuthConfig.build/1","doc":"Builds a `t:ExFTP.Auth.DigestAuthConfig.t/0` from a map\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.DigestAuthConfig.html#build/1"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.DigestAuthConfig.build/1","doc":"* **m** :: A map to build into a `t:ExFTP.Auth.DigestAuthConfig.t/0`\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.DigestAuthConfig.html#build/1-params"},{"type":"type","title":"ExFTP.Auth.DigestAuthConfig.t/0","doc":"The **authenticator_config** value for `ExFTP.Auth.DigestAuth`\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.DigestAuthConfig.html#t:t/0"},{"type":"type","title":"🏷️ Required Keys - ExFTP.Auth.DigestAuthConfig.t/0","doc":"* **login_url** :: `t:ExFTP.Auth.Common.login_url/0`","ref":"ExFTP.Auth.DigestAuthConfig.html#t:t/0-required-keys"},{"type":"type","title":"🏷️ Optional Keys - ExFTP.Auth.DigestAuthConfig.t/0","doc":"* **login_method** :: `t:ExFTP.Auth.Common.login_method/0`\n  * **authenticated_url** :: `t:ExFTP.Auth.Common.authenticated_url/0`\n  * **authenticated_method** :: `t:ExFTP.Auth.Common.authenticated_method/0`\n  * **authenticated_ttl_ms** :: `t:ExFTP.Auth.Common.authenticated_ttl_ms/0`\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.DigestAuthConfig.html#t:t/0-optional-keys"},{"type":"module","title":"ExFTP.Auth.WebhookAuthConfig","doc":"A module describing the **authenticator_config** value for `ExFTP.Auth.WebhookAuth`\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.WebhookAuthConfig.html"},{"type":"module","title":"👀 See Also - ExFTP.Auth.WebhookAuthConfig","doc":"* `ExFTP.Auth.WebhookAuth`\n  * `ExFTP.Authenticator`","ref":"ExFTP.Auth.WebhookAuthConfig.html#module-see-also"},{"type":"module","title":"📖 Resources - ExFTP.Auth.WebhookAuthConfig","doc":"* 📖 [RFC 959](https://www.rfc-editor.org/rfc/rfc959){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Auth.WebhookAuthConfig.html#module-resources"},{"type":"function","title":"ExFTP.Auth.WebhookAuthConfig.build/1","doc":"Builds a `t:ExFTP.Auth.WebhookAuthConfig.t/0` from a map\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.WebhookAuthConfig.html#build/1"},{"type":"function","title":"🏷️ Params - ExFTP.Auth.WebhookAuthConfig.build/1","doc":"* **m** :: A map to build into a `t:ExFTP.Auth.WebhookAuthConfig.t/0`\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.WebhookAuthConfig.html#build/1-params"},{"type":"type","title":"ExFTP.Auth.WebhookAuthConfig.password_hash_type/0","doc":"A hash algorithm to use on the password supplied by the client.\n\nUsed by `ExFTP.Auth.WebhookAuth.login/2`\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.WebhookAuthConfig.html#t:password_hash_type/0"},{"type":"type","title":"👀 See Also - ExFTP.Auth.WebhookAuthConfig.password_hash_type/0","doc":"* `ExFTP.Auth.WebhookAuth.login/2`\n\n\n<!-- tabs-close -->","ref":"ExFTP.Auth.WebhookAuthConfig.html#t:password_hash_type/0-see-also"},{"type":"type","title":"ExFTP.Auth.WebhookAuthConfig.t/0","doc":"The **authenticator_config** value for `ExFTP.Auth.WebhookAuth`\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.WebhookAuthConfig.html#t:t/0"},{"type":"type","title":"🏷️ Required Keys - ExFTP.Auth.WebhookAuthConfig.t/0","doc":"* **login_url** :: `t:ExFTP.Auth.Common.login_url/0`","ref":"ExFTP.Auth.WebhookAuthConfig.html#t:t/0-required-keys"},{"type":"type","title":"🏷️ Optional Keys - ExFTP.Auth.WebhookAuthConfig.t/0","doc":"* **login_method** :: `t:ExFTP.Auth.Common.login_method/0`\n  * **password_hash_type** :: `t:password_hash_type/0`\n  * **authenticated_url** :: `t:ExFTP.Auth.Common.authenticated_url/0`\n  * **authenticated_method** :: `t:ExFTP.Auth.Common.authenticated_url/0`\n  * **authenticated_ttl_ms** :: `t:ExFTP.Auth.Common.authenticated_ttl_ms/0`\n\n<!-- tabs-open -->","ref":"ExFTP.Auth.WebhookAuthConfig.html#t:t/0-optional-keys"},{"type":"module","title":"ExFTP.Storage.FileConnector","doc":"When **storage_connector** is `ExFTP.Storage.FileConnector`, this connector will use the file system of where\nit is running.\n\nThis is the out-of-the-box behavior you'd expect from any FTP server.\n\n> #### 🔒 Security {: .warning}\n>\n> Currently, there is no file access management per user.\n>\n> Authenticated users perform file system actions as the same file system user as the FTP server\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.FileConnector.html"},{"type":"module","title":"⚙️ Configuration - ExFTP.Storage.FileConnector","doc":"*Keys*\n\n* **storage_connector**  == `ExFTP.Storage.FileConnector`\n* **storage_config**  == `%{}`\n\n*Example*\n\n```elixir\n  %{\n    # ... ,\n    storage_connector: ExFTP.Storage.FileConnector,\n    storage_config: %{}\n  }\n```","ref":"ExFTP.Storage.FileConnector.html#module-configuration"},{"type":"module","title":"👀 See Also - ExFTP.Storage.FileConnector","doc":"* `ExFTP.StorageConnector`","ref":"ExFTP.Storage.FileConnector.html#module-see-also"},{"type":"module","title":"📖 Resources - ExFTP.Storage.FileConnector","doc":"* 📖 [RFC 959](https://www.rfc-editor.org/rfc/rfc959){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.FileConnector.html#module-resources"},{"type":"function","title":"ExFTP.Storage.FileConnector.create_write_func/3","doc":"Create a function/1 that writes a **stream** to storage\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.FileConnector.html#create_write_func/3"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.FileConnector.create_write_func/3","doc":"* **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`\n  * **opts** :: list of options","ref":"ExFTP.Storage.FileConnector.html#create_write_func/3-params"},{"type":"function","title":"💻 Examples - ExFTP.Storage.FileConnector.create_write_func/3","doc":"```elixir\n    @impl StorageConnector\n    def create_write_func(path, connector_state, opts \\ []) do\n      fn stream ->\n        fs = File.stream!(path)\n\n        try do\n          _ =\n            stream\n            |> chunk_stream(opts)\n            |> Enum.into(fs)\n\n          {:ok, connector_state}\n        rescue\n          _ ->\n            {:error, \"Failed to transfer\"}\n        end\n      end\n    end\n```\n\n<!-- tabs-close -->","ref":"ExFTP.Storage.FileConnector.html#create_write_func/3-examples"},{"type":"function","title":"ExFTP.Storage.FileConnector.delete_directory/2","doc":"Deletes a given directory\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.FileConnector.html#delete_directory/2"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.FileConnector.delete_directory/2","doc":"* **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.FileConnector.html#delete_directory/2-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Storage.FileConnector.delete_directory/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, connector_state}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, err}\n  ```","ref":"ExFTP.Storage.FileConnector.html#delete_directory/2-returns"},{"type":"function","title":"💻 Examples - ExFTP.Storage.FileConnector.delete_directory/2","doc":"iex> alias ExFTP.Storage.FileConnector\n    iex> connector_state = %{current_working_directory: \"/\"}\n    iex> dir_to_make = File.cwd!() |> Path.join(\"new_dir\")\n    iex> {:ok, connector_state} = FileConnector.make_directory(dir_to_make, connector_state)\n    iex> dir_to_rm = dir_to_make\n    iex> {:ok, connector_state} = FileConnector.delete_directory(dir_to_rm, connector_state)\n    iex> FileConnector.directory_exists?(dir_to_rm, connector_state)\n    false","ref":"ExFTP.Storage.FileConnector.html#delete_directory/2-examples"},{"type":"function","title":"👀 See Also - ExFTP.Storage.FileConnector.delete_directory/2","doc":"* `c:ExFTP.StorageConnector.delete_directory/2`","ref":"ExFTP.Storage.FileConnector.html#delete_directory/2-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.FileConnector.delete_directory/2","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.FileConnector.html#delete_directory/2-resources"},{"type":"function","title":"ExFTP.Storage.FileConnector.delete_file/2","doc":"Deletes a given file\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.FileConnector.html#delete_file/2"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.FileConnector.delete_file/2","doc":"* **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.FileConnector.html#delete_file/2-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Storage.FileConnector.delete_file/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, connector_state}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, err}\n  ```","ref":"ExFTP.Storage.FileConnector.html#delete_file/2-returns"},{"type":"function","title":"👀 See Also - ExFTP.Storage.FileConnector.delete_file/2","doc":"* `c:ExFTP.StorageConnector.delete_file/2`","ref":"ExFTP.Storage.FileConnector.html#delete_file/2-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.FileConnector.delete_file/2","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.FileConnector.html#delete_file/2-resources"},{"type":"function","title":"ExFTP.Storage.FileConnector.directory_exists?/2","doc":"Whether a given path is an existing directory\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.FileConnector.html#directory_exists?/2"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.FileConnector.directory_exists?/2","doc":"* **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.FileConnector.html#directory_exists?/2-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Storage.FileConnector.directory_exists?/2","doc":"**✅ On Success**\n\n  ```elixir\n  `true` or `false`\n  ```","ref":"ExFTP.Storage.FileConnector.html#directory_exists?/2-returns"},{"type":"function","title":"💻 Examples - ExFTP.Storage.FileConnector.directory_exists?/2","doc":"iex> alias ExFTP.Storage.FileConnector\n    iex> FileConnector.directory_exists?(\"/tmp\", %{current_working_directory: \"/\"})\n    true\n    iex> FileConnector.directory_exists?(\"/does-not-exist\", %{current_working_directory: \"/\"})\n    false","ref":"ExFTP.Storage.FileConnector.html#directory_exists?/2-examples"},{"type":"function","title":"👀 See Also - ExFTP.Storage.FileConnector.directory_exists?/2","doc":"* `c:ExFTP.StorageConnector.directory_exists?/2`","ref":"ExFTP.Storage.FileConnector.html#directory_exists?/2-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.FileConnector.directory_exists?/2","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.FileConnector.html#directory_exists?/2-resources"},{"type":"function","title":"ExFTP.Storage.FileConnector.get_content/2","doc":"Returns a stream to read the raw bytes of an object specified by a given path\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.FileConnector.html#get_content/2"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.FileConnector.get_content/2","doc":"* **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.FileConnector.html#get_content/2-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Storage.FileConnector.get_content/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, data}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, err}\n  ```","ref":"ExFTP.Storage.FileConnector.html#get_content/2-returns"},{"type":"function","title":"💻 Examples - ExFTP.Storage.FileConnector.get_content/2","doc":"iex> alias ExFTP.Storage.FileConnector\n    iex> connector_state = %{current_working_directory: \"/\"}\n    iex> file_to_get_content = File.cwd!() |> File.ls!() |> Enum.filter(&String.contains?(&1,\".\")) |> hd()\n    iex> path = Path.join(File.cwd!(), file_to_get_content)\n    iex> {:ok, _data} = FileConnector.get_content(path, connector_state)","ref":"ExFTP.Storage.FileConnector.html#get_content/2-examples"},{"type":"function","title":"👀 See Also - ExFTP.Storage.FileConnector.get_content/2","doc":"* `c:ExFTP.StorageConnector.get_content/2`","ref":"ExFTP.Storage.FileConnector.html#get_content/2-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.FileConnector.get_content/2","doc":"* 📖 [RFC 959 (page-30)](https://www.rfc-editor.org/rfc/rfc959#page-30){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.FileConnector.html#get_content/2-resources"},{"type":"function","title":"ExFTP.Storage.FileConnector.get_content_info/2","doc":"Returns a `t:ExFTP.StorageConnector.content_info/0` representing a given path\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.FileConnector.html#get_content_info/2"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.FileConnector.get_content_info/2","doc":"* **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.FileConnector.html#get_content_info/2-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Storage.FileConnector.get_content_info/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, %{...}}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, err}\n  ```","ref":"ExFTP.Storage.FileConnector.html#get_content_info/2-returns"},{"type":"function","title":"💻 Examples - ExFTP.Storage.FileConnector.get_content_info/2","doc":"iex> alias ExFTP.Storage.FileConnector\n    iex> connector_state = %{current_working_directory: \"/\"}\n    iex> file_to_get_info = File.cwd!() |> File.ls!() |> hd()\n    iex> path = Path.join(File.cwd!(), file_to_get_info)\n    iex> {:ok, _content_info} = FileConnector.get_content_info(path, connector_state)","ref":"ExFTP.Storage.FileConnector.html#get_content_info/2-examples"},{"type":"function","title":"👀 See Also - ExFTP.Storage.FileConnector.get_content_info/2","doc":"* `t:ExFTP.StorageConnector.content_info/0`\n  * `c:ExFTP.StorageConnector.get_content_info/2`\n  * `get_directory_contents/2`","ref":"ExFTP.Storage.FileConnector.html#get_content_info/2-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.FileConnector.get_content_info/2","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.FileConnector.html#get_content_info/2-resources"},{"type":"function","title":"ExFTP.Storage.FileConnector.get_directory_contents/2","doc":"Returns a list of `t:ExFTP.StorageConnector.content_info/0` representing each object in a given directory\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.FileConnector.html#get_directory_contents/2"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.FileConnector.get_directory_contents/2","doc":"* **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.FileConnector.html#get_directory_contents/2-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Storage.FileConnector.get_directory_contents/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, [%{...}, ...]}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, err}\n  ```","ref":"ExFTP.Storage.FileConnector.html#get_directory_contents/2-returns"},{"type":"function","title":"💻 Examples - ExFTP.Storage.FileConnector.get_directory_contents/2","doc":"iex> alias ExFTP.Storage.FileConnector\n    iex> connector_state = %{current_working_directory: \"/\"}\n    iex> dir = File.cwd!()\n    iex> {:ok, _content_infos} = FileConnector.get_directory_contents(dir, connector_state)","ref":"ExFTP.Storage.FileConnector.html#get_directory_contents/2-examples"},{"type":"function","title":"👀 See Also - ExFTP.Storage.FileConnector.get_directory_contents/2","doc":"* `t:ExFTP.StorageConnector.content_info/0`\n  * `c:ExFTP.StorageConnector.get_directory_contents/2`","ref":"ExFTP.Storage.FileConnector.html#get_directory_contents/2-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.FileConnector.get_directory_contents/2","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.FileConnector.html#get_directory_contents/2-resources"},{"type":"function","title":"ExFTP.Storage.FileConnector.get_working_directory/1","doc":"Returns the current working directory\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.FileConnector.html#get_working_directory/1"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.FileConnector.get_working_directory/1","doc":"* **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.FileConnector.html#get_working_directory/1-params"},{"type":"function","title":"💻 Examples - ExFTP.Storage.FileConnector.get_working_directory/1","doc":"iex> alias ExFTP.Storage.FileConnector\n    iex> FileConnector.get_working_directory(%{current_working_directory: \"/\"})\n    \"/\"","ref":"ExFTP.Storage.FileConnector.html#get_working_directory/1-examples"},{"type":"function","title":"👀 See Also - ExFTP.Storage.FileConnector.get_working_directory/1","doc":"* `c:ExFTP.StorageConnector.get_working_directory/1`","ref":"ExFTP.Storage.FileConnector.html#get_working_directory/1-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.FileConnector.get_working_directory/1","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.FileConnector.html#get_working_directory/1-resources"},{"type":"function","title":"ExFTP.Storage.FileConnector.make_directory/2","doc":"Creates a directory, given a path\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.FileConnector.html#make_directory/2"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.FileConnector.make_directory/2","doc":"* **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.FileConnector.html#make_directory/2-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Storage.FileConnector.make_directory/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, connector_state}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, err}\n  ```","ref":"ExFTP.Storage.FileConnector.html#make_directory/2-returns"},{"type":"function","title":"💻 Examples - ExFTP.Storage.FileConnector.make_directory/2","doc":"iex> alias ExFTP.Storage.FileConnector\n    iex> connector_state = %{current_working_directory: \"/\"}\n    iex> dir_to_make = File.cwd!() |> Path.join(\"new_dir\")\n    iex> {:ok, connector_state} = FileConnector.make_directory(dir_to_make, connector_state)\n    iex> FileConnector.directory_exists?(dir_to_make, connector_state)\n    true\n    iex> dir_to_rm = dir_to_make\n    iex> {:ok, connector_state} = FileConnector.delete_directory(dir_to_rm, connector_state)\n    iex> FileConnector.directory_exists?(dir_to_rm, connector_state)\n    false","ref":"ExFTP.Storage.FileConnector.html#make_directory/2-examples"},{"type":"function","title":"👀 See Also - ExFTP.Storage.FileConnector.make_directory/2","doc":"* `c:ExFTP.StorageConnector.make_directory/2`","ref":"ExFTP.Storage.FileConnector.html#make_directory/2-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.FileConnector.make_directory/2","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.FileConnector.html#make_directory/2-resources"},{"type":"module","title":"ExFTP.Storage.S3Connector","doc":"When **storage_connector** is `ExFTP.Storage.S3Connector`, this connector will use any S3-compatible storage provider.\n\nUnderneath the hood, ex_ftp is using `ExAws.S3`, so you'll need that configured properly.\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.S3Connector.html"},{"type":"module","title":"⚙️ Configuration - ExFTP.Storage.S3Connector","doc":"*Keys*\n\n* **storage_connector**  == `ExFTP.Storage.FileConnector`\n* **storage_config**  :: `t:ExFTP.Storage.S3ConnectorConfig.t/0`\n\n*Example*\n\n```elixir\n  %{\n    # ... ,\n    storage_connector: ExFTP.Storage.S3Connector,\n    storage_config: %{\n        # the `/` path of the FTP server will point to s3://{my-storage-bucket}/\n        storage_bucket: \"my-storage-bucket\"\n    }\n}\n```","ref":"ExFTP.Storage.S3Connector.html#module-configuration"},{"type":"module","title":"👀 See Also - ExFTP.Storage.S3Connector","doc":"* `ExFTP.StorageConnector`","ref":"ExFTP.Storage.S3Connector.html#module-see-also"},{"type":"module","title":"📖 Resources - ExFTP.Storage.S3Connector","doc":"* 📖 [RFC 959](https://www.rfc-editor.org/rfc/rfc959){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.S3Connector.html#module-resources"},{"type":"function","title":"ExFTP.Storage.S3Connector.create_write_func/3","doc":"Create a function/1 that writes a **stream** to storage\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.S3Connector.html#create_write_func/3"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.S3Connector.create_write_func/3","doc":"* **path** :: `t:path/0`\n  * **connector_state** :: `t:connector_state/0`\n  * **opts** :: list of options\n\n<!-- tabs-close -->","ref":"ExFTP.Storage.S3Connector.html#create_write_func/3-params"},{"type":"function","title":"ExFTP.Storage.S3Connector.delete_directory/2","doc":"Deletes a given directory\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.S3Connector.html#delete_directory/2"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.S3Connector.delete_directory/2","doc":"* **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.S3Connector.html#delete_directory/2-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Storage.S3Connector.delete_directory/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, connector_state}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, err}\n  ```","ref":"ExFTP.Storage.S3Connector.html#delete_directory/2-returns"},{"type":"function","title":"💻 Examples - ExFTP.Storage.S3Connector.delete_directory/2","doc":"iex> alias ExFTP.Storage.S3Connector\n    iex> connector_state = %{current_working_directory: \"/\"}\n    iex> dir_to_make = \"/new_dir\"\n    iex> {:ok, connector_state} = S3Connector.make_directory(dir_to_make, connector_state)\n    iex> dir_to_rm = dir_to_make\n    iex> {:ok, connector_state} = S3Connector.delete_directory(dir_to_rm, connector_state)\n    iex> S3Connector.directory_exists?(dir_to_rm, connector_state)\n    false","ref":"ExFTP.Storage.S3Connector.html#delete_directory/2-examples"},{"type":"function","title":"👀 See Also - ExFTP.Storage.S3Connector.delete_directory/2","doc":"* `c:ExFTP.StorageConnector.delete_directory/2`","ref":"ExFTP.Storage.S3Connector.html#delete_directory/2-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.S3Connector.delete_directory/2","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.S3Connector.html#delete_directory/2-resources"},{"type":"function","title":"ExFTP.Storage.S3Connector.delete_file/2","doc":"Deletes a given file\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.S3Connector.html#delete_file/2"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.S3Connector.delete_file/2","doc":"* **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.S3Connector.html#delete_file/2-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Storage.S3Connector.delete_file/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, connector_state}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, err}\n  ```","ref":"ExFTP.Storage.S3Connector.html#delete_file/2-returns"},{"type":"function","title":"👀 See Also - ExFTP.Storage.S3Connector.delete_file/2","doc":"* `c:ExFTP.StorageConnector.delete_file/2`","ref":"ExFTP.Storage.S3Connector.html#delete_file/2-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.S3Connector.delete_file/2","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.S3Connector.html#delete_file/2-resources"},{"type":"function","title":"ExFTP.Storage.S3Connector.directory_exists?/2","doc":"Whether a given path is an existing directory\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.S3Connector.html#directory_exists?/2"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.S3Connector.directory_exists?/2","doc":"* **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.S3Connector.html#directory_exists?/2-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Storage.S3Connector.directory_exists?/2","doc":"**✅ On Success**\n\n  ```elixir\n  `true` or `false`\n  ```","ref":"ExFTP.Storage.S3Connector.html#directory_exists?/2-returns"},{"type":"function","title":"💻 Examples - ExFTP.Storage.S3Connector.directory_exists?/2","doc":"iex> alias ExFTP.Storage.S3Connector\n    iex> S3Connector.directory_exists?(\"/\", %{current_working_directory: \"/\"})\n    true\n    iex> S3Connector.directory_exists?(\"/does-not-exist\", %{current_working_directory: \"/\"})\n    false","ref":"ExFTP.Storage.S3Connector.html#directory_exists?/2-examples"},{"type":"function","title":"👀 See Also - ExFTP.Storage.S3Connector.directory_exists?/2","doc":"* `c:ExFTP.StorageConnector.directory_exists?/2`","ref":"ExFTP.Storage.S3Connector.html#directory_exists?/2-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.S3Connector.directory_exists?/2","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.S3Connector.html#directory_exists?/2-resources"},{"type":"function","title":"ExFTP.Storage.S3Connector.get_content/2","doc":"Returns a stream to read the raw bytes of an object specified by a given path\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.S3Connector.html#get_content/2"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.S3Connector.get_content/2","doc":"* **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.S3Connector.html#get_content/2-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Storage.S3Connector.get_content/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, data}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, err}\n  ```","ref":"ExFTP.Storage.S3Connector.html#get_content/2-returns"},{"type":"function","title":"👀 See Also - ExFTP.Storage.S3Connector.get_content/2","doc":"* `c:ExFTP.StorageConnector.get_content/2`","ref":"ExFTP.Storage.S3Connector.html#get_content/2-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.S3Connector.get_content/2","doc":"* 📖 [RFC 959 (page-30)](https://www.rfc-editor.org/rfc/rfc959#page-30){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.S3Connector.html#get_content/2-resources"},{"type":"function","title":"ExFTP.Storage.S3Connector.get_content_info/2","doc":"Returns a `t:ExFTP.StorageConnector.content_info/0` representing a given path\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.S3Connector.html#get_content_info/2"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.S3Connector.get_content_info/2","doc":"* **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.S3Connector.html#get_content_info/2-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Storage.S3Connector.get_content_info/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, %{...}}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, err}\n  ```","ref":"ExFTP.Storage.S3Connector.html#get_content_info/2-returns"},{"type":"function","title":"👀 See Also - ExFTP.Storage.S3Connector.get_content_info/2","doc":"* `t:ExFTP.StorageConnector.content_info/0`\n  * `c:ExFTP.StorageConnector.get_content_info/2`\n  * `get_directory_contents/2`","ref":"ExFTP.Storage.S3Connector.html#get_content_info/2-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.S3Connector.get_content_info/2","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.S3Connector.html#get_content_info/2-resources"},{"type":"function","title":"ExFTP.Storage.S3Connector.get_directory_contents/2","doc":"Returns a list of `t:ExFTP.StorageConnector.content_info/0` representing each object in a given directory\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.S3Connector.html#get_directory_contents/2"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.S3Connector.get_directory_contents/2","doc":"* **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.S3Connector.html#get_directory_contents/2-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Storage.S3Connector.get_directory_contents/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, [%{...}, ...]}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, err}\n  ```","ref":"ExFTP.Storage.S3Connector.html#get_directory_contents/2-returns"},{"type":"function","title":"💻 Examples - ExFTP.Storage.S3Connector.get_directory_contents/2","doc":"iex> alias ExFTP.Storage.S3Connector\n    iex> connector_state = %{current_working_directory: \"/\"}\n    iex> dir = \"/\"\n    iex> {:ok, _content_infos} = S3Connector.get_directory_contents(dir, connector_state)","ref":"ExFTP.Storage.S3Connector.html#get_directory_contents/2-examples"},{"type":"function","title":"👀 See Also - ExFTP.Storage.S3Connector.get_directory_contents/2","doc":"* `t:ExFTP.StorageConnector.content_info/0`\n  * `c:ExFTP.StorageConnector.get_directory_contents/2`","ref":"ExFTP.Storage.S3Connector.html#get_directory_contents/2-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.S3Connector.get_directory_contents/2","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.S3Connector.html#get_directory_contents/2-resources"},{"type":"function","title":"ExFTP.Storage.S3Connector.get_working_directory/1","doc":"Returns the current working directory\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.S3Connector.html#get_working_directory/1"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.S3Connector.get_working_directory/1","doc":"* **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.S3Connector.html#get_working_directory/1-params"},{"type":"function","title":"💻 Examples - ExFTP.Storage.S3Connector.get_working_directory/1","doc":"iex> alias ExFTP.Storage.S3Connector\n    iex> S3Connector.get_working_directory(%{current_working_directory: \"/\"})\n    \"/\"","ref":"ExFTP.Storage.S3Connector.html#get_working_directory/1-examples"},{"type":"function","title":"👀 See Also - ExFTP.Storage.S3Connector.get_working_directory/1","doc":"* `c:ExFTP.StorageConnector.get_working_directory/1`","ref":"ExFTP.Storage.S3Connector.html#get_working_directory/1-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.S3Connector.get_working_directory/1","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.S3Connector.html#get_working_directory/1-resources"},{"type":"function","title":"ExFTP.Storage.S3Connector.make_directory/2","doc":"Creates a directory, given a path\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.S3Connector.html#make_directory/2"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.S3Connector.make_directory/2","doc":"* **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.S3Connector.html#make_directory/2-params"},{"type":"function","title":"⤵️ Returns - ExFTP.Storage.S3Connector.make_directory/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, connector_state}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, err}\n  ```","ref":"ExFTP.Storage.S3Connector.html#make_directory/2-returns"},{"type":"function","title":"💻 Examples - ExFTP.Storage.S3Connector.make_directory/2","doc":"iex> alias ExFTP.Storage.S3Connector\n    iex> connector_state = %{current_working_directory: \"/\"}\n    iex> dir_to_make = \"/new_dir/\"\n    iex> {:ok, connector_state} = S3Connector.make_directory(dir_to_make, connector_state)\n    iex> S3Connector.directory_exists?(dir_to_make, connector_state)\n    true","ref":"ExFTP.Storage.S3Connector.html#make_directory/2-examples"},{"type":"function","title":"👀 See Also - ExFTP.Storage.S3Connector.make_directory/2","doc":"* `c:ExFTP.StorageConnector.make_directory/2`","ref":"ExFTP.Storage.S3Connector.html#make_directory/2-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.S3Connector.make_directory/2","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.S3Connector.html#make_directory/2-resources"},{"type":"behaviour","title":"ExFTP.StorageConnector","doc":"A behaviour defining a Storage Connector.\n\nStorage Connectors are used by the FTP interface to interact with a particular type of storage.\n\n<!-- tabs-open -->","ref":"ExFTP.StorageConnector.html"},{"type":"behaviour","title":"👀 See Also - ExFTP.StorageConnector","doc":"* `ExFTP.Storage.FileConnector`","ref":"ExFTP.StorageConnector.html#module-see-also"},{"type":"behaviour","title":"📖 Resources - ExFTP.StorageConnector","doc":"* 📖 [RFC 959](https://www.rfc-editor.org/rfc/rfc959){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.StorageConnector.html#module-resources"},{"type":"type","title":"ExFTP.StorageConnector.connector_state/0","doc":"State held onto by the server and modified by the `StorageConnector`.\n\nIt may be used by the storage connector to keep stateful values.\n\n<!-- tabs-open -->","ref":"ExFTP.StorageConnector.html#t:connector_state/0"},{"type":"type","title":"⚠️ Reminders - ExFTP.StorageConnector.connector_state/0","doc":"> #### Special Keys {: .tip}\n>\n>   * `current_working_directory:` `t:String.t/0` should always exist. It represents the \"directory\" we're operating from","ref":"ExFTP.StorageConnector.html#t:connector_state/0-reminders"},{"type":"type","title":"📖 Resources - ExFTP.StorageConnector.connector_state/0","doc":"* 📖 [RFC 959](https://www.rfc-editor.org/rfc/rfc959){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.StorageConnector.html#t:connector_state/0-resources"},{"type":"type","title":"ExFTP.StorageConnector.content_info/0","doc":"Information about a given file, directory, or symlink\n\n<!-- tabs-open -->","ref":"ExFTP.StorageConnector.html#t:content_info/0"},{"type":"type","title":"🏷️ Keys - ExFTP.StorageConnector.content_info/0","doc":"* **filename** :: `t:String.t/0` e.g \"my_file.txt\", \"my_dir/\" or \"my_sym_link -> target/\"\n  * **modified_datetime** :: `t:DateTime.t/0`\n  * **size** :: `integer` e.g  1000\n  * **access** :: `:read | :write | :read_write | :none`\n  * **type** :: `:directory | :symlink | :file`","ref":"ExFTP.StorageConnector.html#t:content_info/0-keys"},{"type":"type","title":"👀 See Also - ExFTP.StorageConnector.content_info/0","doc":"* `c:get_content_info/2`\n  * `c:get_directory_contents/2`","ref":"ExFTP.StorageConnector.html#t:content_info/0-see-also"},{"type":"type","title":"📖 Resources - ExFTP.StorageConnector.content_info/0","doc":"* 📖 [RFC 959](https://www.rfc-editor.org/rfc/rfc959){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.StorageConnector.html#t:content_info/0-resources"},{"type":"callback","title":"ExFTP.StorageConnector.create_write_func/3","doc":"Create a function/1 that writes a **stream** to storage\n\n<!-- tabs-open -->","ref":"ExFTP.StorageConnector.html#c:create_write_func/3"},{"type":"callback","title":"🏷️ Params - ExFTP.StorageConnector.create_write_func/3","doc":"* **path** :: `t:path/0`\n  * **connector_state** :: `t:connector_state/0`\n  * **opts** :: list of options","ref":"ExFTP.StorageConnector.html#c:create_write_func/3-params"},{"type":"callback","title":"💻 Examples - ExFTP.StorageConnector.create_write_func/3","doc":"```elixir\n    @impl StorageConnector\n    def create_write_func(path, connector_state, opts \\ []) do\n      fn stream ->\n        fs = File.stream!(path)\n\n        try do\n          _ =\n            stream\n            |> chunk_stream(opts)\n            |> Enum.into(fs)\n\n          {:ok, connector_state}\n        rescue\n          _ ->\n            {:error, \"Failed to transfer\"}\n        end\n      end\n    end\n```\n\n<!-- tabs-close -->","ref":"ExFTP.StorageConnector.html#c:create_write_func/3-examples"},{"type":"callback","title":"ExFTP.StorageConnector.delete_directory/2","doc":"Deletes a given directory\n\n<!-- tabs-open -->","ref":"ExFTP.StorageConnector.html#c:delete_directory/2"},{"type":"callback","title":"🏷️ Params - ExFTP.StorageConnector.delete_directory/2","doc":"* **path** :: `t:path/0`\n  * **connector_state** :: `t:connector_state/0`","ref":"ExFTP.StorageConnector.html#c:delete_directory/2-params"},{"type":"callback","title":"⤵️ Returns - ExFTP.StorageConnector.delete_directory/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, connector_state}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, err}\n  ```","ref":"ExFTP.StorageConnector.html#c:delete_directory/2-returns"},{"type":"callback","title":"💻 Examples - ExFTP.StorageConnector.delete_directory/2","doc":"iex> alias ExFTP.Storage.FileConnector\n    iex> connector_state = %{current_working_directory: \"/\"}\n    iex> dir_to_make = File.cwd!() |> Path.join(\"new_dir\")\n    iex> {:ok, connector_state} = FileConnector.make_directory(dir_to_make, connector_state)\n    iex> dir_to_rm = dir_to_make\n    iex> {:ok, connector_state} = FileConnector.delete_directory(dir_to_rm, connector_state)\n    iex> FileConnector.directory_exists?(dir_to_rm, connector_state)\n    false","ref":"ExFTP.StorageConnector.html#c:delete_directory/2-examples"},{"type":"callback","title":"📖 Resources - ExFTP.StorageConnector.delete_directory/2","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.StorageConnector.html#c:delete_directory/2-resources"},{"type":"callback","title":"ExFTP.StorageConnector.delete_file/2","doc":"Deletes a given file\n\n<!-- tabs-open -->","ref":"ExFTP.StorageConnector.html#c:delete_file/2"},{"type":"callback","title":"🏷️ Params - ExFTP.StorageConnector.delete_file/2","doc":"* **path** :: `t:path/0`\n  * **connector_state** :: `t:connector_state/0`","ref":"ExFTP.StorageConnector.html#c:delete_file/2-params"},{"type":"callback","title":"⤵️ Returns - ExFTP.StorageConnector.delete_file/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, connector_state}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, err}\n  ```","ref":"ExFTP.StorageConnector.html#c:delete_file/2-returns"},{"type":"callback","title":"📖 Resources - ExFTP.StorageConnector.delete_file/2","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.StorageConnector.html#c:delete_file/2-resources"},{"type":"callback","title":"ExFTP.StorageConnector.directory_exists?/2","doc":"Whether a given path is an existing directory\n\n<!-- tabs-open -->","ref":"ExFTP.StorageConnector.html#c:directory_exists?/2"},{"type":"callback","title":"🏷️ Params - ExFTP.StorageConnector.directory_exists?/2","doc":"* **path** :: `t:path/0`\n  * **connector_state** :: `t:connector_state/0`","ref":"ExFTP.StorageConnector.html#c:directory_exists?/2-params"},{"type":"callback","title":"⤵️ Returns - ExFTP.StorageConnector.directory_exists?/2","doc":"**✅ On Success**\n\n  ```elixir\n  `true` or `false`\n  ```","ref":"ExFTP.StorageConnector.html#c:directory_exists?/2-returns"},{"type":"callback","title":"💻 Examples - ExFTP.StorageConnector.directory_exists?/2","doc":"iex> alias ExFTP.Storage.FileConnector\n    iex> FileConnector.directory_exists?(\"/tmp\", %{current_working_directory: \"/\"})\n    true\n    iex> FileConnector.directory_exists?(\"/does-not-exist\", %{current_working_directory: \"/\"})\n    false","ref":"ExFTP.StorageConnector.html#c:directory_exists?/2-examples"},{"type":"callback","title":"📖 Resources - ExFTP.StorageConnector.directory_exists?/2","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.StorageConnector.html#c:directory_exists?/2-resources"},{"type":"callback","title":"ExFTP.StorageConnector.get_content/2","doc":"Returns a stream to read the raw bytes of an object specified by a given path\n\n<!-- tabs-open -->","ref":"ExFTP.StorageConnector.html#c:get_content/2"},{"type":"callback","title":"🏷️ Params - ExFTP.StorageConnector.get_content/2","doc":"* **path** :: `t:path/0`\n  * **connector_state** :: `t:connector_state/0`","ref":"ExFTP.StorageConnector.html#c:get_content/2-params"},{"type":"callback","title":"⤵️ Returns - ExFTP.StorageConnector.get_content/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, data}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, err}\n  ```","ref":"ExFTP.StorageConnector.html#c:get_content/2-returns"},{"type":"callback","title":"💻 Examples - ExFTP.StorageConnector.get_content/2","doc":"iex> alias ExFTP.Storage.FileConnector\n    iex> connector_state = %{current_working_directory: \"/\"}\n    iex> file_to_get_content = File.cwd!() |> File.ls!() |> Enum.filter(&String.contains?(&1,\".\")) |> hd()\n    iex> path = Path.join(File.cwd!(), file_to_get_content)\n    iex> {:ok, _data} = FileConnector.get_content(path, connector_state)","ref":"ExFTP.StorageConnector.html#c:get_content/2-examples"},{"type":"callback","title":"📖 Resources - ExFTP.StorageConnector.get_content/2","doc":"* 📖 [RFC 959 (page-30)](https://www.rfc-editor.org/rfc/rfc959#page-30){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.StorageConnector.html#c:get_content/2-resources"},{"type":"callback","title":"ExFTP.StorageConnector.get_content_info/2","doc":"Returns `t:content_info/0` representing a single object in a given directory\n\n<!-- tabs-open -->","ref":"ExFTP.StorageConnector.html#c:get_content_info/2"},{"type":"callback","title":"🏷️ Params - ExFTP.StorageConnector.get_content_info/2","doc":"* **path** :: `t:path/0`\n  * **connector_state** :: `t:connector_state/0`","ref":"ExFTP.StorageConnector.html#c:get_content_info/2-params"},{"type":"callback","title":"⤵️ Returns - ExFTP.StorageConnector.get_content_info/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, %{...}}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, err}\n  ```","ref":"ExFTP.StorageConnector.html#c:get_content_info/2-returns"},{"type":"callback","title":"💻 Examples - ExFTP.StorageConnector.get_content_info/2","doc":"iex> alias ExFTP.Storage.FileConnector\n    iex> connector_state = %{current_working_directory: \"/\"}\n    iex> file_to_get_info = File.cwd!() |> File.ls!() |> hd()\n    iex> path = Path.join(File.cwd!(), file_to_get_info)\n    iex> {:ok, content_info} = FileConnector.get_content_info(path, connector_state)","ref":"ExFTP.StorageConnector.html#c:get_content_info/2-examples"},{"type":"callback","title":"📖 Resources - ExFTP.StorageConnector.get_content_info/2","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.StorageConnector.html#c:get_content_info/2-resources"},{"type":"callback","title":"ExFTP.StorageConnector.get_directory_contents/2","doc":"Returns a list of `t:content_info/0` representing each object in a given directory\n\n<!-- tabs-open -->","ref":"ExFTP.StorageConnector.html#c:get_directory_contents/2"},{"type":"callback","title":"🏷️ Params - ExFTP.StorageConnector.get_directory_contents/2","doc":"* **path** :: `t:path/0`\n  * **connector_state** :: `t:connector_state/0`","ref":"ExFTP.StorageConnector.html#c:get_directory_contents/2-params"},{"type":"callback","title":"⤵️ Returns - ExFTP.StorageConnector.get_directory_contents/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, [%{...}, ...]}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, err}\n  ```","ref":"ExFTP.StorageConnector.html#c:get_directory_contents/2-returns"},{"type":"callback","title":"💻 Examples - ExFTP.StorageConnector.get_directory_contents/2","doc":"iex> alias ExFTP.Storage.FileConnector\n    iex> connector_state = %{current_working_directory: \"/\"}\n    iex> dir = File.cwd!()\n    iex> {:ok, _content_infos} = FileConnector.get_directory_contents(dir, connector_state)","ref":"ExFTP.StorageConnector.html#c:get_directory_contents/2-examples"},{"type":"callback","title":"📖 Resources - ExFTP.StorageConnector.get_directory_contents/2","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.StorageConnector.html#c:get_directory_contents/2-resources"},{"type":"callback","title":"ExFTP.StorageConnector.get_working_directory/1","doc":"Returns the current working directory\n\n<!-- tabs-open -->","ref":"ExFTP.StorageConnector.html#c:get_working_directory/1"},{"type":"callback","title":"🏷️ Params - ExFTP.StorageConnector.get_working_directory/1","doc":"* **connector_state** :: `t:connector_state/0`","ref":"ExFTP.StorageConnector.html#c:get_working_directory/1-params"},{"type":"callback","title":"⤵️ Returns - ExFTP.StorageConnector.get_working_directory/1","doc":"**✅ On Success**\n\n  ```elixir\n  (The working directory)\n  ```","ref":"ExFTP.StorageConnector.html#c:get_working_directory/1-returns"},{"type":"callback","title":"💻 Examples - ExFTP.StorageConnector.get_working_directory/1","doc":"iex> alias ExFTP.Storage.FileConnector\n    iex> FileConnector.get_working_directory(%{current_working_directory: \"/\"})\n    \"/\"","ref":"ExFTP.StorageConnector.html#c:get_working_directory/1-examples"},{"type":"callback","title":"⚠️ Reminders - ExFTP.StorageConnector.get_working_directory/1","doc":"> #### Doesn't the connector_state already have it? {: .tip}\n>\n> Most Storage Connectors will just return what's already in the **connector_state**.\n> However, this method is implemented just in case a Connector has a different way of\n> determining the current working directory.","ref":"ExFTP.StorageConnector.html#c:get_working_directory/1-reminders"},{"type":"callback","title":"📖 Resources - ExFTP.StorageConnector.get_working_directory/1","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.StorageConnector.html#c:get_working_directory/1-resources"},{"type":"callback","title":"ExFTP.StorageConnector.make_directory/2","doc":"Creates a directory, given a path\n\n<!-- tabs-open -->","ref":"ExFTP.StorageConnector.html#c:make_directory/2"},{"type":"callback","title":"🏷️ Params - ExFTP.StorageConnector.make_directory/2","doc":"* **path** :: `t:path/0`\n  * **connector_state** :: `t:connector_state/0`","ref":"ExFTP.StorageConnector.html#c:make_directory/2-params"},{"type":"callback","title":"⤵️ Returns - ExFTP.StorageConnector.make_directory/2","doc":"**✅ On Success**\n\n  ```elixir\n  {:ok, connector_state}\n  ```\n  **❌ On Failure**\n\n   ```elixir\n  {:error, err}\n  ```","ref":"ExFTP.StorageConnector.html#c:make_directory/2-returns"},{"type":"callback","title":"💻 Examples - ExFTP.StorageConnector.make_directory/2","doc":"iex> alias ExFTP.Storage.FileConnector\n    iex> connector_state = %{current_working_directory: \"/\"}\n    iex> dir_to_make = File.cwd!() |> Path.join(\"new_dir\")\n    iex> {:ok, connector_state} = FileConnector.make_directory(dir_to_make, connector_state)\n    iex> FileConnector.directory_exists?(dir_to_make, connector_state)\n    true\n    iex> {:ok, _connector_state} = FileConnector.delete_directory(dir_to_make, connector_state)\n    iex> FileConnector.directory_exists?(dir_to_rm, connector_state)\n    false","ref":"ExFTP.StorageConnector.html#c:make_directory/2-examples"},{"type":"callback","title":"📖 Resources - ExFTP.StorageConnector.make_directory/2","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.StorageConnector.html#c:make_directory/2-resources"},{"type":"type","title":"ExFTP.StorageConnector.path/0","doc":"A string representing a file path (e.g `\"/path/to/file.txt\"` or `\"/path/to/dir/\"`)\n\n<!-- tabs-open -->","ref":"ExFTP.StorageConnector.html#t:path/0"},{"type":"type","title":"⚠️ Reminders - ExFTP.StorageConnector.path/0","doc":"> #### Relative paths {: .tip}\n>\n> The Server will ensure all paths sent to the connector are *absolute* paths,\n> so you don't need to worry about handling relative paths, or `..` notations","ref":"ExFTP.StorageConnector.html#t:path/0-reminders"},{"type":"type","title":"📖 Resources - ExFTP.StorageConnector.path/0","doc":"* 📖 [RFC 959](https://www.rfc-editor.org/rfc/rfc959){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.StorageConnector.html#t:path/0-resources"},{"type":"type","title":"ExFTP.StorageConnector.socket/0","doc":"A Port representing a socket to communicate with an FTP client.\n\n<!-- tabs-open -->","ref":"ExFTP.StorageConnector.html#t:socket/0"},{"type":"type","title":"⚠️ Reminders - ExFTP.StorageConnector.socket/0","doc":"> #### Sockets are everywhere {: .tip}\n>\n> This socket represents the TCP connection between the FTP Server and the client (often through port 21)\n>\n> While related, this socket is not a PASV socket, which is a negotiated, temporary socket\n> for sending or receiving data.","ref":"ExFTP.StorageConnector.html#t:socket/0-reminders"},{"type":"type","title":"📖 Resources - ExFTP.StorageConnector.socket/0","doc":"* 📖 [RFC 959](https://www.rfc-editor.org/rfc/rfc959){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n<!-- tabs-close -->","ref":"ExFTP.StorageConnector.html#t:socket/0-resources"},{"type":"module","title":"ExFTP.Storage.S3ConnectorConfig","doc":"A module describing the **authenticator_config** value for `ExFTP.Storage.S3Connector`\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.S3ConnectorConfig.html"},{"type":"module","title":"👀 See Also - ExFTP.Storage.S3ConnectorConfig","doc":"* `ExFTP.Storage.S3Connector`\n  * `ExFTP.StorageConnector`","ref":"ExFTP.Storage.S3ConnectorConfig.html#module-see-also"},{"type":"module","title":"📖 Resources - ExFTP.Storage.S3ConnectorConfig","doc":"* 📖 [RFC 959](https://www.rfc-editor.org/rfc/rfc959){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.S3ConnectorConfig.html#module-resources"},{"type":"function","title":"ExFTP.Storage.S3ConnectorConfig.build/1","doc":"Builds a `t:ExFTP.Storage.S3ConnectorConfig.t/0` from a map\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.S3ConnectorConfig.html#build/1"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.S3ConnectorConfig.build/1","doc":"* **m** :: A map to build into a `t:ExFTP.Storage.S3ConnectorConfig.t/0`\n\n<!-- tabs-close -->","ref":"ExFTP.Storage.S3ConnectorConfig.html#build/1-params"},{"type":"type","title":"ExFTP.Storage.S3ConnectorConfig.storage_bucket/0","doc":"A bucket to use as the root directory of the FTP server.","ref":"ExFTP.Storage.S3ConnectorConfig.html#t:storage_bucket/0"},{"type":"type","title":"ExFTP.Storage.S3ConnectorConfig.t/0","doc":"The **storage_config** value for `ExFTP.Storage.S3Connector`\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.S3ConnectorConfig.html#t:t/0"},{"type":"type","title":"🏷️ Required Keys - ExFTP.Storage.S3ConnectorConfig.t/0","doc":"* **storage_bucket** :: `t:ExFTP.Storage.S3ConnectorConfig.storage_bucket/0`\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.S3ConnectorConfig.html#t:t/0-required-keys"},{"type":"module","title":"ExFTP.Common","doc":"This module contains shared functions between `ExFTP.Worker` and `ExFTP.Storage.Common`","ref":"ExFTP.Common.html"},{"type":"function","title":"ExFTP.Common.send_resp/3","doc":"Responds to the FTP client","ref":"ExFTP.Common.html#send_resp/3"},{"type":"module","title":"ExFTP.Storage.Common","doc":"A module covering the low-level FTP responses\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.Common.html"},{"type":"module","title":"👀 See Also - ExFTP.Storage.Common","doc":"* `ExFTP.Worker`","ref":"ExFTP.Storage.Common.html#module-see-also"},{"type":"module","title":"📖 Resources - ExFTP.Storage.Common","doc":"* 📖 [RFC 959](https://www.rfc-editor.org/rfc/rfc959){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.Common.html#module-resources"},{"type":"function","title":"ExFTP.Storage.Common.chunk_stream/2","doc":"Chunks a stream into `chunk_size` pieces for upload\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.Common.html#chunk_stream/2"},{"type":"function","title":"🏷️ Params - ExFTP.Storage.Common.chunk_stream/2","doc":"* **stream** :: A Stream to read from\n  * **opts** :: `t:ExFTP.StorageConnector.path/0`\n    * **chunk_size** :: `t:pos_integer()`\n\n<!-- tabs-close -->","ref":"ExFTP.Storage.Common.html#chunk_stream/2-params"},{"type":"function","title":"ExFTP.Storage.Common.cwd/1","doc":"Responds to FTP's `CWD` command\n\n> #### RFC 959: CHANGE WORKING DIRECTORY (CWD) {: .tip}\n> This command allows the user to work with a different\n> directory or dataset for file storage or retrieval without\n> altering their login or accounting information.  Transfer\n> parameters are similarly unchanged.  The argument is a\n> pathname specifying a directory or other system dependent\n> file group designator.\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.Common.html#cwd/1"},{"type":"function","title":"🏷️ Server State - ExFTP.Storage.Common.cwd/1","doc":"* **storage_connector** :: `ExFTP.StorageConnector`\n  * **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **socket** :: `t:ExFTP.StorageConnector.socket/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.Common.html#cwd/1-server-state"},{"type":"function","title":"👀 See Also - ExFTP.Storage.Common.cwd/1","doc":"* `c:ExFTP.StorageConnector.get_working_directory/1`","ref":"ExFTP.Storage.Common.html#cwd/1-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.Common.cwd/1","doc":"* 📖 [RFC 959 (page-26)](https://www.rfc-editor.org/rfc/rfc959#page-26){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.Common.html#cwd/1-resources"},{"type":"function","title":"ExFTP.Storage.Common.dele/1","doc":"Responds to FTP's `DELE` command\n\n> #### RFC 959: DELETE (DELE) {: .tip}\n> This command causes the file specified in the pathname to be\n> deleted at the server site.  If an extra level of protection\n> is desired (such as the query, \"Do you really wish to\n> delete?\"), it should be provided by the user-FTP process.\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.Common.html#dele/1"},{"type":"function","title":"🏷️ Server State - ExFTP.Storage.Common.dele/1","doc":"* **storage_connector** :: `ExFTP.StorageConnector`\n  * **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **socket** :: `t:ExFTP.StorageConnector.socket/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.Common.html#dele/1-server-state"},{"type":"function","title":"👀 See Also - ExFTP.Storage.Common.dele/1","doc":"* `c:ExFTP.StorageConnector.delete_file/2`","ref":"ExFTP.Storage.Common.html#dele/1-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.Common.dele/1","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.Common.html#dele/1-resources"},{"type":"function","title":"ExFTP.Storage.Common.list/1","doc":"Responds to FTP's `LIST` command\n\n> #### RFC 959: LIST (LIST) {: .tip}\n> This command causes a list to be sent from the server to the\n> passive DTP.  If the pathname specifies a directory or other\n> group of files, the server should transfer a list of files\n> in the specified directory.  If the pathname specifies a\n> file then the server should send current information on the\n> file.  A null argument implies the user's current working or\n> default directory.  The data transfer is over the data\n> connection in type ASCII or type EBCDIC.  (The user must\n> ensure that the TYPE is appropriately ASCII or EBCDIC).\n> Since the information on a file may vary widely from system\n> to system, this information may be hard to use automatically\n> in a program, but may be quite useful to a human user.\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.Common.html#list/1"},{"type":"function","title":"🏷️ Server State - ExFTP.Storage.Common.list/1","doc":"* **storage_connector** :: `ExFTP.StorageConnector`\n  * **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **socket** :: `t:ExFTP.StorageConnector.socket/0`\n  * **pasv** :: `t:pasv_socket/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`\n  * **include_hidden** :: `t:boolean/0` - Whether to include hidden files/dirs","ref":"ExFTP.Storage.Common.html#list/1-server-state"},{"type":"function","title":"👀 See Also - ExFTP.Storage.Common.list/1","doc":"* `c:ExFTP.StorageConnector.get_directory_contents/2`","ref":"ExFTP.Storage.Common.html#list/1-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.Common.list/1","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.Common.html#list/1-resources"},{"type":"function","title":"ExFTP.Storage.Common.mkd/1","doc":"Responds to FTP's `MKD` command\n\n> #### RFC 959: MAKE DIRECTORY (MKD) {: .tip}\n> This command causes the directory specified in the pathname\n> to be created as a directory (if the pathname is absolute)\n> or as a subdirectory of the current working directory (if\n> the pathname is relative).\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.Common.html#mkd/1"},{"type":"function","title":"🏷️ Server State - ExFTP.Storage.Common.mkd/1","doc":"* **storage_connector** :: `ExFTP.StorageConnector`\n  * **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **socket** :: `t:ExFTP.StorageConnector.socket/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.Common.html#mkd/1-server-state"},{"type":"function","title":"👀 See Also - ExFTP.Storage.Common.mkd/1","doc":"* `c:ExFTP.StorageConnector.make_directory/2`","ref":"ExFTP.Storage.Common.html#mkd/1-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.Common.mkd/1","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.Common.html#mkd/1-resources"},{"type":"function","title":"ExFTP.Storage.Common.nlst/1","doc":"Responds to FTP's `NLST` command\n\n> #### RFC 959: NAME LIST (NLST) {: .tip}\n> This command causes a directory listing to be sent from\n> server to user site.  The pathname should specify a\n> directory or other system-specific file group descriptor; a\n> null argument implies the current directory.  The server\n> will return a stream of names of files and no other\n> information.  The data will be transferred in ASCII or\n> EBCDIC type over the data connection as valid pathname\n> strings separated by   or  .  (Again the user must\n> ensure that the TYPE is correct.)  This command is intended\n> to return information that can be used by a program to\n> further process the files automatically.  For example, in\n> the implementation of a \"multiple get\" function.\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.Common.html#nlst/1"},{"type":"function","title":"🏷️ Server State - ExFTP.Storage.Common.nlst/1","doc":"* **storage_connector** :: `ExFTP.StorageConnector`\n  * **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **socket** :: `t:ExFTP.StorageConnector.socket/0`\n  * **pasv** :: `t:pasv_socket/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`\n  * **include_hidden** :: `t:boolean/0` - Whether to include hidden files/dirs","ref":"ExFTP.Storage.Common.html#nlst/1-server-state"},{"type":"function","title":"👀 See Also - ExFTP.Storage.Common.nlst/1","doc":"* `c:ExFTP.StorageConnector.get_directory_contents/2`","ref":"ExFTP.Storage.Common.html#nlst/1-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.Common.nlst/1","doc":"* 📖 [RFC 959 (page-33)](https://www.rfc-editor.org/rfc/rfc959#page-33){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.Common.html#nlst/1-resources"},{"type":"type","title":"ExFTP.Storage.Common.pasv_socket/0","doc":"A `t:port/0` representing a temporary, negotiated passive socket to communicate with an FTP client.\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.Common.html#t:pasv_socket/0"},{"type":"type","title":"⚠️ Reminders - ExFTP.Storage.Common.pasv_socket/0","doc":"> #### Sockets are everywhere {: .tip}\n>\n> This socket represents a temporary TCP connection between the FTP Server and the client\n>\n> While related, this passive socket is not the normal socket, which is often on port 21.","ref":"ExFTP.Storage.Common.html#t:pasv_socket/0-reminders"},{"type":"type","title":"📖 Resources - ExFTP.Storage.Common.pasv_socket/0","doc":"* 📖 [RFC 959 (page-28)](https://www.rfc-editor.org/rfc/rfc959#page-28){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n<!-- tabs-close -->","ref":"ExFTP.Storage.Common.html#t:pasv_socket/0-resources"},{"type":"function","title":"ExFTP.Storage.Common.prepare/1","doc":"Takes a map and ensures the keys are atoms and use the correct conventions","ref":"ExFTP.Storage.Common.html#prepare/1"},{"type":"function","title":"ExFTP.Storage.Common.pwd/1","doc":"Responds to FTP's `PWD` command\n\n> #### RFC 959: PRINT WORKING DIRECTORY (PWD) {: .tip}\n> This command causes the name of the current working\n> directory to be returned in the reply.\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.Common.html#pwd/1"},{"type":"function","title":"🏷️ Server State - ExFTP.Storage.Common.pwd/1","doc":"* **storage_connector** :: `ExFTP.StorageConnector`\n  * **socket** :: `t:ExFTP.StorageConnector.socket/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.Common.html#pwd/1-server-state"},{"type":"function","title":"👀 See Also - ExFTP.Storage.Common.pwd/1","doc":"* `c:ExFTP.StorageConnector.get_working_directory/1`","ref":"ExFTP.Storage.Common.html#pwd/1-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.Common.pwd/1","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.Common.html#pwd/1-resources"},{"type":"function","title":"ExFTP.Storage.Common.retr/1","doc":"Responds to FTP's `RETR` command\n\n> #### RFC 959: RETRIEVE (RETR) {: .tip}\n> This command causes the server-DTP to transfer a copy of the\n> file, specified in the pathname, to the server- or user-DTP\n> at the other end of the data connection.  The status and\n> contents of the file at the server site shall be unaffected.\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.Common.html#retr/1"},{"type":"function","title":"🏷️ Server State - ExFTP.Storage.Common.retr/1","doc":"* **storage_connector** :: `ExFTP.StorageConnector`\n  * **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **socket** :: `t:ExFTP.StorageConnector.socket/0`\n  * **pasv** :: `t:pasv_socket/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.Common.html#retr/1-server-state"},{"type":"function","title":"👀 See Also - ExFTP.Storage.Common.retr/1","doc":"* `c:ExFTP.StorageConnector.get_content/2`","ref":"ExFTP.Storage.Common.html#retr/1-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.Common.retr/1","doc":"* 📖 [RFC 959 (page-30)](https://www.rfc-editor.org/rfc/rfc959#page-30){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.Common.html#retr/1-resources"},{"type":"function","title":"ExFTP.Storage.Common.rmd/1","doc":"Responds to FTP's `RMD` command\n\n> #### RFC 959: REMOVE DIRECTORY (RMD) {: .tip}\n> This command causes the directory specified in the pathname\n> to be removed as a directory (if the pathname is absolute)\n> or as a subdirectory of the current working directory (if\n> the pathname is relative).\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.Common.html#rmd/1"},{"type":"function","title":"🏷️ Server State - ExFTP.Storage.Common.rmd/1","doc":"* **storage_connector** :: `ExFTP.StorageConnector`\n  * **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **socket** :: `t:ExFTP.StorageConnector.socket/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.Common.html#rmd/1-server-state"},{"type":"function","title":"👀 See Also - ExFTP.Storage.Common.rmd/1","doc":"* `c:ExFTP.StorageConnector.delete_directory/2`","ref":"ExFTP.Storage.Common.html#rmd/1-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.Common.rmd/1","doc":"* 📖 [RFC 959 (page-32)](https://www.rfc-editor.org/rfc/rfc959#page-32){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.Common.html#rmd/1-resources"},{"type":"function","title":"ExFTP.Storage.Common.size/1","doc":"Responds to FTP's `SIZE` command\n\n> #### RFC 3659: SIZE OF FILE (SIZE) {: .tip}\n> The FTP command, SIZE OF FILE (SIZE), is used to obtain the transfer\n> size of a file from the server-FTP process.  This is the exact number\n> of octets (8 bit bytes) that would be transmitted over the data\n> connection should that file be transmitted.  This value will change\n> depending on the current STRUcture, MODE, and TYPE of the data\n> connection or of a data connection that would be created were one\n> created now.  Thus, the result of the SIZE command is dependent on\n> the currently established STRU, MODE, and TYPE parameters.\n>\n> The SIZE command returns how many octets would be transferred if the\n> file were to be transferred using the current transfer structure,\n> mode, and type.  This command is normally used in conjunction with\n> the RESTART (REST) command when STORing a file to a remote server in\n> STREAM mode, to determine the restart point.  The server-PI might\n> need to read the partially transferred file, do any appropriate\n> conversion, and count the number of octets that would be generated\n> when sending the file in order to correctly respond to this command.\n> Estimates of the file transfer size MUST NOT be returned; only\n> precise information is acceptable.\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.Common.html#size/1"},{"type":"function","title":"🏷️ Server State - ExFTP.Storage.Common.size/1","doc":"* **storage_connector** :: `ExFTP.StorageConnector`\n  * **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **socket** :: `t:ExFTP.StorageConnector.socket/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.Common.html#size/1-server-state"},{"type":"function","title":"👀 See Also - ExFTP.Storage.Common.size/1","doc":"* `c:ExFTP.StorageConnector.get_content_info/2`","ref":"ExFTP.Storage.Common.html#size/1-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.Common.size/1","doc":"* 📖 [RFC 959](https://www.rfc-editor.org/rfc/rfc959){:target=\"_blank\"}\n  * 📖 [RFC 3659 (section-4)](https://www.rfc-editor.org/rfc/3659#section-4){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.Common.html#size/1-resources"},{"type":"function","title":"ExFTP.Storage.Common.stor/1","doc":"Responds to FTP's `STOR` command\n\n> #### RFC 959: STORE (STOR) {: .tip}\n> This command causes the server-DTP to accept the data\n> transferred via the data connection and to store the data as\n> a file at the server site.  If the file specified in the\n> pathname exists at the server site, then its contents shall\n> be replaced by the data being transferred.  A new file is\n> created at the server site if the file specified in the\n> pathname does not already exist.\n\n<!-- tabs-open -->","ref":"ExFTP.Storage.Common.html#stor/1"},{"type":"function","title":"🏷️ Server State - ExFTP.Storage.Common.stor/1","doc":"* **storage_connector** :: `ExFTP.StorageConnector`\n  * **path** :: `t:ExFTP.StorageConnector.path/0`\n  * **socket** :: `t:ExFTP.StorageConnector.socket/0`\n  * **pasv** :: `t:pasv_socket/0`\n  * **connector_state** :: `t:ExFTP.StorageConnector.connector_state/0`","ref":"ExFTP.Storage.Common.html#stor/1-server-state"},{"type":"function","title":"👀 See Also - ExFTP.Storage.Common.stor/1","doc":"* `c:ExFTP.StorageConnector.get_content/2`","ref":"ExFTP.Storage.Common.html#stor/1-see-also"},{"type":"function","title":"📖 Resources - ExFTP.Storage.Common.stor/1","doc":"* 📖 [RFC 959 (page-30)](https://www.rfc-editor.org/rfc/rfc959#page-30){:target=\"_blank\"}\n  * 📖 [RFC 3659](https://www.rfc-editor.org/rfc/rfc3659){:target=\"_blank\"}\n  * 💬 Contact the maintainer (he's happy to help!)\n    * 👾 [Github: camatcode](https://github.com/camatcode/){:target=\"_blank\"}\n    * 🐘 [Fediverse: @scrum_log@maston.social](https://mastodon.social/@scrum_log){:target=\"_blank\"}\n    \n\n<!-- tabs-close -->","ref":"ExFTP.Storage.Common.html#stor/1-resources"},{"type":"function","title":"ExFTP.Storage.Common.validate_config/1","doc":"Ensures you get a valid config according to a module, **mod**","ref":"ExFTP.Storage.Common.html#validate_config/1"},{"type":"module","title":"ExFTP.Worker","doc":"A module defining a `GenServer` which serves the FTP interface","ref":"ExFTP.Worker.html"},{"type":"function","title":"ExFTP.Worker.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExFTP.Worker.html#child_spec/1"},{"type":"function","title":"ExFTP.Worker.start_link/1","doc":"","ref":"ExFTP.Worker.html#start_link/1"},{"type":"extras","title":"README","doc":"An extendable, lightweight FTP server with cloud integrations already built in","ref":"readme.html"},{"type":"extras","title":"Table of Contents - README","doc":"- [Installation](#installation)\n- [Reckless Quick Start](#reckless-quick-start)\n- [Configuration](#configuration)\n  - [Server Config](#1-server-config)\n  - [Choosing an Authenticator](#2-choose-an-authenticator)\n  - [Choosing a Storage Connector](#3-choose-a-storage-connector)\n- [Authenticators](#authenticators)\n  - [No Auth](#authenticator-no-auth)\n  - [Passthrough Auth](#authenticator-passthrough-auth)\n  - [HTTP Basic Auth](#authenticator-http-basic-auth)\n  - [HTTP Digest Access Auth](#authenticator-http-digest-access-auth)\n  - [Bearer Token Auth](#authenticator-bearer-token-auth)\n  - [Webhook Auth](#authenticator-webhook-auth)\n  - [Custom Auth](#authenticator-custom-auth)\n- [Storage Connectors](#storage-connectors)\n  - [File](#storage-connector-file)\n  - [S3](#storage-connector-s3)\n    - [Using Minio or LocalStack](#using-minio-or-localstack)\n  - [Supabase Storage](#storage-connector-supabase)\n  - [Others through S3Proxy](#storage-connector-others-through-s3proxy)\n  - [Custom Storage Connector](#custom-storage-connector)\n- [Technical Details](#technical-details)\n  - [Supported Commands](#supported-commands)\n  - [Notes about Fly.io](#notes-about-fly.io)\n- [Special Thanks](#special-thanks)","ref":"readme.html#table-of-contents"},{"type":"extras","title":"Installation - README","doc":"Add `:ex_ftp` to your list of deps in `mix.exs`:\n\n```elixir\n{:ex_ftp, \"~> 1.0\"}\n```\n\nThen run `mix deps.get` to install ExFTP and its dependencies.","ref":"readme.html#installation"},{"type":"extras","title":"Reckless Quick Start - README","doc":"* Configure ex_ftp \n  * to use the file system, \n  * start on port 4040, \n  * don't include auth\n\n```elixir\nconfig :ex_ftp,\n  ftp_port: \"FTP_PORT\" |> System.get_env(\"4040\") |> String.to_integer(),\n  min_passive_port: \"MIN_PASSIVE_PORT\" |> System.get_env(\"40002\") |> String.to_integer(),\n  max_passive_port: \"MAX_PASSIVE_PORT\" |> System.get_env(\"40007\") |> String.to_integer(),\n  authenticator: ExFTP.Auth.NoAuth,\n  authenticator_config: %{},\n  storage_connector: ExFTP.Storage.FileConnector\n```\n\n* Run `mix run --no-halt`\n\n```\n17:13:22.110 [info] Accepting connections on port 4040\n```\n\n* Connect using `ftp`\n\n```bash\n➜  ~ ftp localhost -p 4040      \n\nConnected to localhost.\n220 Hello from ExFTP.\nName (localhost:cam): \n331 User name okay, need password.\nPassword: \n502 Command not implemented.\nftp: Login failed\nftp> ls\n229 Entering Extended Passive Mode (|||40002|)\n150 Here comes the directory listing.\nlr--r--r--    1 0        0               7 Feb 16  2024 bin -> usr/bin\ndr--r--r--    1 0        0            4096 May 13  2025 boot\ndr--r--r--    1 0        0            4096 Feb 16  2024 cdrom\ndr--r--r--    1 0        0            4680 May 20  2025 dev\ndr--r--r--    1 0        0           12288 May 19  2025 etc\ndr--r--r--    1 0        0            4096 Mar 25  2025 home\nlr--r--r--    1 0        0               7 Feb 16  2024 lib -> usr/lib\ndr--r--r--    1 0        0            4096 Feb 06  2025 lib32\nlr--r--r--    1 0        0               9 Feb 16  2024 lib64 -> usr/lib64\ndr--r--r--    1 0        0            4096 Feb 06  2025 libx32\nd---r--r--    1 0        0           16384 Feb 16  2024 lost+found\ndr--r--r--    1 0        0            4096 Feb 29  2024 media\ndr--r--r--    1 0        0            4096 Jan 09  2024 mnt\ndrw-r--r--    1 0        0            4096 Apr 24  2025 opt\ndr--r--r--    1 0        0               0 May 02  2025 proc\nd---r--r--    1 0        0            4096 Mar 25  2025 root\ndr--r--r--    1 0        0            1580 May 17  2025 run\nlr--r--r--    1 0        0               8 Feb 16  2024 sbin -> usr/sbin\ndr--r--r--    1 0        0            4096 Jan 09  2024 srv\n----r--r--    1 0        0      2147483648 Feb 16  2024 swapfile\ndr--r--r--    1 0        0               0 May 02  2025 sys\ndr--r--r--    1 0        0            4096 May 19  2025 timeshift\ndrw-r--r--    1 0        0           20480 May 20  2025 tmp\ndr--r--r--    1 0        0            4096 Apr 25  2025 usr\ndr--r--r--    1 0        0            4096 Mar 25  2025 var\n226 Directory send OK.\nftp> ...\n```\n\n* Now, [properly configure it](#configuration).\n\n\n-------","ref":"readme.html#reckless-quick-start"},{"type":"extras","title":"Configuration - README","doc":"","ref":"readme.html#configuration"},{"type":"extras","title":"1. Server Config - README","doc":"Here is a detailed, example configuration.\n\n```elixir\nconfig :ex_ftp,\n  # port to run on\n  ftp_port: 21,\n  # FTP uses temporary, negotiated ports for certain commands called passive ports\n  # Choose the min and max range for these ports\n  # This range would represent how many of these certain commands can run at the same time.\n  # Be aware, too few options could create bottlenecks\n  min_passive_port: System.get_env(\"MIN_PASSIVE_PORT\", \"40002\") |> String.to_integer(),\n  max_passive_port: System.get_env(\"MAX_PASSIVE_PORT\", \"40012\") |> String.to_integer(),\n  # See \"Choose an Authenticator\"\n  authenticator: ExFTP.Auth.BasicAuth,\n  authenticator_config: %{\n    # used to login\n    login_url: \"https://httpbin.dev/basic-auth/\",\n    login_method: :get,\n    # used to verify the user is still considered valid (optional)\n    authenticated_url: \"https://httpbin.dev/hidden-basic-auth/\",\n    authenticated_method: :get,\n    authenticated_ttl_ms: 1000 * 60 * 60\n  },\n  # See \"Choose a Storage Connector\"\n  storage_connector: ExFTP.Storage.FileConnector,\n  storage_config: %{}\n\n```","ref":"readme.html#1-server-config"},{"type":"extras","title":"2. Choose an Authenticator - README","doc":"An `ExFTP.Authenticator` validates credentials when an FTP client sends a `USER` and `PASS` command.\n\nEach authenticator is referenced in the `ex_ftp` config under the `authenticator` key. \n\nAdditionally, many require a map under `authenticator_config`.","ref":"readme.html#2-choose-an-authenticator"},{"type":"extras","title":"3. Choose a Storage Connector - README","doc":"An `ExFTP.StorageConnector` provides access to your chosen storage provider - with the FTP business abstracted away.\n\nEach storage connector is referenced in the `ex_ftp` config under the `storage_connector` key.\n\nAdditionally, many require a map under `storage_config`.\n\n-------","ref":"readme.html#3-choose-a-storage-connector"},{"type":"extras","title":"Authenticators - README","doc":"Below are all the included authenticators.","ref":"readme.html#authenticators"},{"type":"extras","title":"Authenticator: No Auth - README","doc":"> [!WARNING]\n> This is not recommended for any production server.\n\nWhen **authenticator** is `ExFTP.Auth.NoAuth`, this authenticator will completely ignore any supplied credentials and \nassume everything is authenticated.\n\n\n```elixir\nconfig :ex_ftp,\n  #....\n  authenticator: ExFTP.Auth.NoAuth,\n  authenticator_config: %{}\n ```\n\n-------","ref":"readme.html#authenticator-no-auth"},{"type":"extras","title":"Authenticator: Passthrough Auth - README","doc":"> [!WARNING] \n> This is not recommended for any production server.\n\nWhen **authenticator** is `ExFTP.Auth.PassthroughAuth`, this authenticator will require credentials, \nbut accepts any user and password combination who isn't `root`.\n\n```elixir\nconfig :ex_ftp,\n  #....\n  authenticator: ExFTP.Auth.PassthroughAuth,\n  authenticator_config: %{}\n ```\n\n[^ top](#top)\n\n-------","ref":"readme.html#authenticator-passthrough-auth"},{"type":"extras","title":"Authenticator: HTTP Basic Auth - README","doc":"> [!WARNING]  \n> This is not recommended for situations not protected by SSL.\n\nWhen **authenticator** is `ExFTP.Auth.BasicAuth`, this authenticator will call out to an HTTP endpoint that implements \n[HTTP Basic Auth](https://en.wikipedia.org/wiki/Basic_access_authentication) with the user's supplied credentials.\n\n\n```elixir\nconfig :ex_ftp,\n  #....\n  authenticator: ExFTP.Auth.BasicAuth,\n  authenticator_config: %{\n    # used to login\n    login_url: \"https://httpbin.dev/basic-auth/\",\n    login_method: :get,\n    # used to verify the user is still considered valid (optional)\n    authenticated_url: \"https://httpbin.dev/hidden-basic-auth/\",\n    authenticated_method: :get,\n    authenticated_ttl_ms: 1000 * 60 * 60\n  }\n ```\n\nIf the endpoint responds with **HTTP 200**, the user is considered authenticated.\n\nAdditionally, if configured, ex_ftp can call out to a separate endpoint that performs basic auth to check that a user\nis still considered valid.\n\n[^ top](#top)\n\n-------","ref":"readme.html#authenticator-http-basic-auth"},{"type":"extras","title":"Authenticator: HTTP Digest Access Auth - README","doc":"> [!NOTE]  \n> This can be used in situations where SSL is not available, though be warned, Digest Access is considered\n> an obsolete protocol.\n\nWhen **authenticator** is `ExFTP.Auth.DigestAuth`, this authenticator will call out to an HTTP endpoint that\nimplements [HTTP Digest Access Auth](https://en.wikipedia.org/wiki/Digest_access_authentication) with the user's\nsupplied credentials.\n\n\n```elixir\nconfig :ex_ftp,\n  #....\n  authenticator: ExFTP.Auth.DigestAuth,\n  authenticator_config: %{\n    # used to login\n    login_url: \"https://httpbin.dev/basic-auth/\",\n    login_method: :get,\n    # used to verify the user is still considered valid (optional)\n    authenticated_url: \"https://httpbin.dev/hidden-basic-auth/\",\n    authenticated_method: :get,\n    authenticated_ttl_ms: 1000 * 60 * 60\n  }\n ```\n\nIf, after completing the full workflow, the endpoint responds with **HTTP 200**, the user is considered authenticated.\n\nAdditionally, if configured, ex_ftp can call out to a separate endpoint that performs digest auth to check that a user\nis still considered valid.\n\n[^ top](#top)\n\n-------","ref":"readme.html#authenticator-http-digest-access-auth"},{"type":"extras","title":"Authenticator: Bearer Token Auth - README","doc":"> [!NOTE]  \n> This is helpful when the \"user\" is actually a system or process.\n>\n> `username` isn't important for a Bearer token; though a provided username is still held on to.\n\nWhen **authenticator** is `ExFTP.Auth.BearerAuth`, this authenticator will call out to an HTTP endpoint that implements\n[Bearer Tokens](https://swagger.io/docs/specification/v3_0/authentication/bearer-authentication/) with the user's \nsupplied credentials.\n\n```elixir\nconfig :ex_ftp,\n  #....\n  authenticator: ExFTP.Auth.BearerAuth,\n  authenticator_config: %{\n    # used to login\n    login_url: \"https://httpbin.dev/bearer\",\n    login_method: :post,\n    # used to verify the user is still considered valid (optional)\n    authenticated_url: \"https://httpbin.dev/bearer\",\n    authenticated_method: :post,\n    authenticated_ttl_ms: 1000 * 60 * 60\n  }\n ```\n\nIf the endpoint responds with **HTTP 200**, the user is considered authenticated.\n\nAdditionally, if configured, ex_ftp can call out to a separate endpoint that performs bearer auth to check that a user\nis still considered valid.\n\n\n[^ top](#top)\n\n-------","ref":"readme.html#authenticator-bearer-token-auth"},{"type":"extras","title":"Authenticator: Webhook Auth - README","doc":"> [!NOTE]  \n> `password_hash` is the hash of the supplied password using the hashing algorithm dictated by the config.\n\nWhen **authenticator** is `ExFTP.Auth.WebhookAuth`, this authenticator will call out to an HTTP endpoint that accepts\ntwo query parameters: `username` and/or `password_hash`.\n\n\n```elixir\nconfig :ex_ftp,\n  #....\n  authenticator: ExFTP.Auth.WebhookAuth,\n  authenticator_config: %{\n    # used to login\n    login_url: \"https://httpbin.dev/status/200\",\n    login_method: :post,\n    # affects the output of the `password_hash` query parameter\n    # accepts anything that :crypto can handle\n    password_hash_type: :sha256,\n    # used to verify the user is still considered valid (optional)\n    authenticated_url: \"https://httpbin.dev/status/200\",\n    authenticated_method: :post,\n    authenticated_ttl_ms: 1000 * 60 * 60\n  }\n ```\n\nIf the endpoint responds with **HTTP 200**, the user is considered authenticated.\n\nAdditionally, if configured, ex_ftp can call out to a separate endpoint that performs webhook auth to check that a user\nis still considered valid.\n\n[^ top](#top)\n\n-------","ref":"readme.html#authenticator-webhook-auth"},{"type":"extras","title":"Authenticator: Custom Auth - README","doc":"Creating your own Authenticator is simple - just implement the `ExFTP.Authenticator` behaviour.\n\n```elixir\n# SPDX-License-Identifier: Apache-2.0\ndefmodule MyCustomAuth do\n\n  alias ExFTP.Authenticator\n  @behaviour Authenticator\n\n  @impl Authenticator\n  @spec valid_user?(username :: Authenticator.username()) :: boolean\n  def valid_user?(username) do\n        # return true if the username is valid\n        # return false if invalid\n        # this short-circuits bad login requests,\n        # if it would take longer than 5 seconds to validate a username\n        #   then its best to just return true\n        #   as there wouldn't be a performance benefit\n  end\n\n  @impl Authenticator\n  @spec login(\n          password :: Authenticator.password(),\n          authenticator_state :: Authenticator.authenticator_state()\n        ) :: {:ok, Authenticator.authenticator_state()} | {:error, term()}\n  def login(password, authenticator_state) do\n        # authenticator_state may have the key :username\n        # perform initial login\n        # return {:ok, current_authenticator_state} if successful\n        #   authenticator_state is passed around during the session\n        #   your authenticated?/1 may want this method to put \n        #     something about the password in the state\n        # return {:error, anything} if unsuccessful\n  end\n\n  @impl Authenticator\n  @spec authenticated?(\n          authenticator_state :: Authenticator.authenticator_state()\n        ) :: boolean()\n  def authenticated?(authenticator_state), do\n        # re-check that a user is authenticated\n        # return {:ok, current_authenticator_state} if successful\n        #   authenticator_state is passed around during the session\n        # return {:error, anything} if unsuccessful\n  end\nend\n```\n\n[^ top](#top)\n\n-------","ref":"readme.html#authenticator-custom-auth"},{"type":"extras","title":"Storage Connectors - README","doc":"Below are all the included storage connectors.","ref":"readme.html#storage-connectors"},{"type":"extras","title":"Storage Connector: File - README","doc":"When **storage_connector** is `ExFTP.Storage.FileConnector`, this connector will use the file system of where \nit is running.\n\nThis is the out-of-the-box behavior you'd expect from any FTP server.\n\n```elixir\nconfig :ex_ftp,\n  #....\n  storage_connector: ExFTP.Storage.FileConnector,\n  storage_config: %{}\n```\n\n[^ top](#top)\n\n-------","ref":"readme.html#storage-connector-file"},{"type":"extras","title":"Storage Connector: S3 - README","doc":"When **storage_connector** is `ExFTP.Storage.S3Connector`, this connector will use any S3-compatible storage provider.\n\nUnderneath the hood, ex_ftp is using `ExAws.S3`, so you'll need that configured properly.\n\n```elixir\n# ExAws is pretty smart figuring out S3 credentials of the system\n# For me, I had to include the region.\n# Consult the ExAws docs for more\nconfig :ex_aws,\n  region: {:system, \"AWS_REGION\"}\n\nconfig :ex_ftp,\n  #....\n  storage_connector: ExFTP.Storage.S3Connector,\n  storage_config: %{\n    # the `/` path of the FTP server will point to s3://{my-storage-bucket}/\n    storage_bucket: \"my-storage-bucket\"\n  }\n```\n\n#### Using Minio or LocalStack\n\nMinio is a popular open-source, self-hosted alternative to AWS S3. \n\nLocalStack is a popular way to test AWS without connecting to AWS.\n\nThe only difference in config will be how you configure `ExAws`.\n\nHere's an example with minio where we're changing the credentials and endpoint\n\n```elixir\n# Assuming:\n#   we're connecting to a minio @ https://my.minio.example.com:9000/\n#   there exists a $MINIO_ACCESS or $AWS_ACCESS_KEY_ID in system env\n#   there exists a $MINIO_SECRET or $AWS_SECRET_ACCESS_KEY in system env\nconfig :ex_aws,\n  access_key_id: [\n    {:system, \"MINIO_ACCESS\"},\n    {:system, \"AWS_ACCESS_KEY_ID\"},\n    :instance_role\n  ],\n  secret_access_key: [\n    {:system, \"MINIO_SECRET\"},\n    {:system, \"AWS_SECRET_ACCESS_KEY\"},\n    :instance_role\n  ],\n  s3: [\n    scheme: \"https://\",\n    host: \"my.minio.example.com\",\n    port: 9000,\n    region: \"us-east-1\"\n  ]\n\nconfig :ex_ftp,\n  #....\n  storage_connector: ExFTP.Storage.S3Connector,\n  storage_config: %{\n    # the `/` path of the FTP server will point to s3://{my-storage-bucket}/\n    storage_bucket: \"my-storage-bucket\"\n  }\n```\n\n[^ top](#top)\n\n-------","ref":"readme.html#storage-connector-s3"},{"type":"extras","title":"Storage Connector: Supabase - README","doc":"🚧 TODO\n\n[^ top](#top)\n\n-------","ref":"readme.html#storage-connector-supabase"},{"type":"extras","title":"Storage Connector: Others through S3Proxy - README","doc":"For other storage providers (Google Cloud, Azure Storage, etc.), it's probably best to deploy a proxy that translates\nS3 requests into requests to those providers, then use the `ExFTP.Storage.S3Connector` to connect to that proxy.\n\n* See [S3Proxy](https://github.com/gaul/s3proxy?tab=readme-ov-file)\n\n[^ top](#top)\n\n-------","ref":"readme.html#storage-connector-others-through-s3proxy"},{"type":"extras","title":"Custom Storage Connector - README","doc":"Creating your own Storage Connector is simple - just implement the `ExFTP.StorageConnector` behaviour.\n\n```elixir\n# SPDX-License-Identifier: Apache-2.0\ndefmodule MyStorageConnector do\n  @moduledoc false\n\n  @behaviour ExFTP.StorageConnector\n\n  alias ExFTP.StorageConnector\n\n  @impl StorageConnector\n  @spec get_working_directory(connector_state :: StorageConnector.connector_state()) ::\n          String.t()\n  def get_working_directory(%{current_working_directory: cwd} = _connector_state) do\n    # returns the current directory, for most cases this is just a pass through\n    # however, you might want to modify what the current directory is\n    # based on some state\n  end\n\n  @impl StorageConnector\n  @spec directory_exists?(\n          path :: StorageConnector.path(),\n          connector_state :: StorageConnector.connector_state()\n        ) :: boolean\n  def directory_exists?(path, _connector_state) do\n    # Given a path, does this directory exist in storage?\n  end\n\n  @impl StorageConnector\n  @spec make_directory(\n          path :: StorageConnector.path(),\n          connector_state :: StorageConnector.connector_state()\n        ) :: {:ok, StorageConnector.connector_state()} | {:error, term()}\n  def make_directory(path, connector_state) do\n    # Given a path, make a directory\n    # For S3-like connectors, a \"directory\" doesn't really exist\n    #  so those connectors typically keep track of virtual directories\n    #  that we're created by user during the session\n    #  if they're unused, they aren't persisted.\n  end\n\n  @impl StorageConnector\n  @spec delete_directory(\n          path :: StorageConnector.path(),\n          connector_state :: StorageConnector.connector_state()\n        ) :: {:ok, StorageConnector.connector_state()} | {:error, term()}\n  def delete_directory(path, connector_state) do\n    # Give a path, delete the directory\n  end\n\n  @impl StorageConnector\n  @spec delete_file(\n          path :: StorageConnector.path(),\n          connector_state :: StorageConnector.connector_state()\n        ) :: {:ok, StorageConnector.connector_state()} | {:error, term()}\n  def delete_file(path, connector_state) do\n    # Give a path, delete the file\n  end\n\n  @impl StorageConnector\n  @spec get_directory_contents(\n          path :: StorageConnector.path(),\n          connector_state :: StorageConnector.connector_state()\n        ) ::\n          {:ok, [StorageConnector.content_info()]} | {:error, term()}\n  def get_directory_contents(path, connector_state) do\n    # returns a list of content_infos\n    # the model for them was inspired by File.lstat()\n    # Have a look at StorageConnector.content_info type\n  end\n\n  @impl StorageConnector\n  @spec get_content_info(\n          path :: StorageConnector.path(),\n          connector_state :: StorageConnector.connector_state()\n        ) ::\n          {:ok, StorageConnector.content_info()} | {:error, term()}\n  def get_content_info(path, _connector_state) do\n    # given a path, return information on the file/directory there\n    # Have a look at StorageConnector.content_info type\n  end\n\n  @impl StorageConnector\n  @spec get_content(\n          path :: StorageConnector.path(),\n          connector_state :: StorageConnector.connector_state()\n        ) :: {:ok, any()} | {:error, term()}\n  def get_content(path, _connector_state) do\n    # Return a {:ok, stream} of path\n  end\n\n  @impl StorageConnector\n  @spec create_write_func(\n          path :: StorageConnector.path(),\n          connector_state :: StorageConnector.connector_state(),\n          opts :: list()\n        ) :: function()\n  def create_write_func(path, connector_state, opts \\\\ []) do\n    # Return a function that will write `stream` to your storage at path\n    # e.g \n    # fn stream ->\n    #  fs = File.stream!(path)\n    #\n    #  try do\n    #    _ =\n    #      stream\n    #      |> chunk_stream(opts)\n    #      |> Enum.into(fs)\n    #\n    #    {:ok, connector_state}\n    #  rescue\n    #    _ ->\n    #      {:error, \"Failed to transfer\"}\n    #  end\n    #end\n  end\nend\n```\n\n[^ top](#top)\n\n-------","ref":"readme.html#custom-storage-connector"},{"type":"extras","title":"Technical Details - README","doc":"","ref":"readme.html#technical-details"},{"type":"extras","title":"Supported Commands - README","doc":"- General\n  - `QUIT`\n  - `SYST`\n  - `TYPE  `\n  - `PASV`\n  - `EPSV`\n  - `EPRT  `\n- Auth\n  - `USER  `\n  - `PASS  `\n- Storage \n  - `PWD`\n  - `CDUP`\n  - `CWD  `\n  - `MKD  `\n  - `RMD  `\n  - `DELE  `\n  - `LIST`\n    - `LIST -a`\n    - `LIST -a  `\n    - `LIST  `\n  - `NLST`\n    - `NLST -a`\n    - `NLST -a  `\n    - `NLST  `\n  - `RETR  `\n  - `SIZE  `\n  - `STOR  `\n\nSee `ExFTP.Storage.Common` for more information.","ref":"readme.html#supported-commands"},{"type":"extras","title":"Notes about Fly.io - README","doc":"If you're wanting to deploy onto Fly.io, you'll quickly discover an issue with passive ports.\n\nFly wants you to enumerate all ports that your server will use, fine; however, it takes the assumption\nthat these ports will be open *on start* and will *remain* open. \n\nFTP passive ports are temporary and negotiated. Fly hates this and assumes something is going wrong.\n\nBe careful.\n\n[^ top](#top)\n\n-----","ref":"readme.html#notes-about-fly-io"},{"type":"extras","title":"Special Thanks - README","doc":"The initial funding for this code came from [StudioCMS.io](https://studiocms.io/).\n\nIts first closed-source implementation came from [Jake Stover](https://github.com/jwstover) and expanded by the \nentire team at StudioCMS.\n\nFurthermore, StudioCMS's leadership allowed me to clean it up, generalize it, and open source it.\n\nThanks!\n\n[^ top](#top)","ref":"readme.html#special-thanks"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.1"}}